(*
 * Copyright  2003 Cadence Berkeley Labs. Cadence Design Systems.
 *
 *)

(* Given two predicates p1 and p2:,
   1) Verify that they are inconsistent, and
   2) Return an interpolant for (p1,p2).

   Raise the exception SAT if the formulas are consistent.

   Exception SAT has two parameters representing
   the satisfying assignment:

      1) list of literals over predicate symbols
      2) list of expression/value pairs

   Note: values of expressions are rationals, since we don't
   fully implement integer arithmetic.

*)

open FociDag
  
exception SAT of (predicate list) * ((Concrete.concrete * Num.num) list)
  
val interpolate : (predicate * predicate) -> predicate
val interpolateN : (predicate list) -> (predicate list)
val interpolateN_symm : (predicate list) -> (predicate list)

(* Interpolation with selection of theory.
   
   "interpolateN_in_theory list symm theory" refutes the predicate list,
   and returns an interpolant (or raises SAT). If symm is true, a symmetric
   interpolant is returned. For the parameter "theory", see FociDag. *)

val interpolateN_in_theory : (predicate list) -> bool -> theory -> (predicate list)

val theoryClauses : unit -> predicate list list
