/*
 * Copyright  2005 Cadence Berkeley Labs. Cadence Design Systems.
 *
 */

/* C interface functions to foci */

#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/callback.h>

#include <stdio.h>

typedef struct foci_term {value obj;} foci_term;
typedef struct foci_term_list {value obj;} foci_term_list;

typedef struct foci_pred {value obj;} foci_pred;
typedef struct foci_pred_list {value obj;} foci_pred_list;

#define FOCI_C_INTF_INTERNAL
#include "foci_c_intf_body.h"

/* some accounting */

int foci_c_intf_objects = 0;

static void my_register_global_root(value *p){
  register_global_root(p);
  foci_c_intf_objects++;
  /* printf("r:0x%x\n",p); */
}

static void my_remove_global_root(value *p){
  remove_global_root(p);
  foci_c_intf_objects--;
  /* printf("u:0x%x\n",p); */
}

#undef register_global_root
#define register_global_root my_register_global_root
#undef remove_global_root
#define remove_global_root my_remove_global_root

/* entries into the ocaml code */

value *closure_foci_constant;
value *closure_foci_atom;
value *closure_foci_variable;
value *closure_foci_application;
value *closure_foci_coeff;
value *closure_foci_sum;
value *closure_foci_equality;
value *closure_foci_leq;
value *closure_foci_and;
value *closure_foci_or;
value *closure_foci_not;
value *closure_foci_ite;
value *closure_foci_interpolate;
value *closure_foci_term_type;
value *closure_foci_pred_type;
value *closure_foci_constant_value;
value *closure_foci_atom_name;
value *closure_foci_variable_name;
value *closure_foci_application_name;
value *closure_foci_application_args;
value *closure_foci_coeff_value;
value *closure_foci_coeff_arg;
value *closure_foci_sum_args;
value *closure_foci_equality_left;
value *closure_foci_equality_right;
value *closure_foci_leq_left;
value *closure_foci_leq_right;
value *closure_foci_and_args;
value *closure_foci_or_args;
value *closure_foci_not_arg;
value *closure_foci_ite_if;
value *closure_foci_ite_then;
value *closure_foci_ite_else;
value *closure_foci_term_print;
value *closure_foci_pred_print;
value *closure_foci_print_stats;
value *closure_foci_gc;

void foci_c_intf_init(){
  char *argv[2];
  argv[0] = "foci_c_intf";
  argv[1] = 0;
  caml_startup(argv);
  
  closure_foci_constant = caml_named_value("foci_constant");
  closure_foci_atom = caml_named_value("foci_atom");
  closure_foci_variable = caml_named_value("foci_variable");
  closure_foci_application = caml_named_value("foci_application");
  closure_foci_coeff = caml_named_value("foci_coeff");
  closure_foci_sum = caml_named_value("foci_sum");
  closure_foci_equality = caml_named_value("foci_equality");
  closure_foci_leq = caml_named_value("foci_leq");
  closure_foci_and = caml_named_value("foci_and");
  closure_foci_or = caml_named_value("foci_or");
  closure_foci_not = caml_named_value("foci_not");
  closure_foci_ite = caml_named_value("foci_ite");
  closure_foci_interpolate = caml_named_value("foci_interpolate");
  closure_foci_term_type = caml_named_value("foci_term_type");
  closure_foci_pred_type = caml_named_value("foci_pred_type");
  closure_foci_constant_value = caml_named_value("foci_constant_value");
  closure_foci_atom_name = caml_named_value("foci_atom_name");
  closure_foci_variable_name = caml_named_value("foci_variable_name");
  closure_foci_application_name = caml_named_value("foci_application_name");
  closure_foci_application_args = caml_named_value("foci_application_args");
  closure_foci_coeff_value = caml_named_value("foci_coeff_value");
  closure_foci_coeff_arg = caml_named_value("foci_coeff_arg");
  closure_foci_sum_args = caml_named_value("foci_sum_args");
  closure_foci_equality_left = caml_named_value("foci_equality_left");
  closure_foci_equality_right = caml_named_value("foci_equality_right");
  closure_foci_leq_left = caml_named_value("foci_leq_left");
  closure_foci_leq_right = caml_named_value("foci_leq_right");
  closure_foci_and_args = caml_named_value("foci_and_args");
  closure_foci_or_args = caml_named_value("foci_or_args");
  closure_foci_not_arg = caml_named_value("foci_not_arg");
  closure_foci_ite_if = caml_named_value("foci_ite_if");
  closure_foci_ite_then = caml_named_value("foci_ite_then");
  closure_foci_ite_else = caml_named_value("foci_ite_else");
  closure_foci_term_print = caml_named_value("foci_term_print");
  closure_foci_pred_print = caml_named_value("foci_pred_print");
  closure_foci_print_stats = caml_named_value("foci_print_stats");
  closure_foci_gc = caml_named_value("foci_gc");
}

static value list_of_expr_array(int size, const foci_term *arr){
  CAMLparam0();
  CAMLlocal2(res,temp);
  
  /* turn the clause into an ml list */
  
  res = Val_int(0); /* empty list */
  
  {
    const foci_term *p = arr+size-1;
    int count;
    for(count = size; count > 0; count--, p--){
      temp = res;
      res = alloc(2, 0);
      Store_field(res, 0, p->obj);
      Store_field(res, 1, temp);
    }
  }
  
  CAMLreturn (res);
}

static int list_length(value l){
  int res = 0;
  for(; Is_block(l); res++, l = Field(l,1));
  return res;
}

static int *list_to_expr_array(value l, int size, foci_pred *arr){
  int i;
  
  for(i = 0; Is_block(l) && i < size; i++, l = Field(l,1)){
    arr[i].obj = Field(l,0);
    register_global_root(&(arr[i].obj));
  }
  return;
}

void foci_term_null(foci_term *e){
  e->obj = 0;
}

void foci_pred_null(foci_pred *e){
  e->obj = 0;
}

void foci_constant(int c, foci_term *e){
  e->obj = callback(*closure_foci_constant, Val_int(c));
  register_global_root(&(e->obj));
  return;
}

void foci_atom(const char *name, foci_pred *e){
  e->obj = callback(*closure_foci_atom, copy_string(name));
  register_global_root(&(e->obj));
  return;
}

void foci_variable(const char *name, foci_term *e){
  e->obj = callback(*closure_foci_variable, copy_string(name));
  register_global_root(&(e->obj));
  return;
}

void foci_application(const char *name, int nargs, const foci_term *args, foci_term *e){
  e->obj = callback2(*closure_foci_application, copy_string(name), list_of_expr_array(nargs,args));
  register_global_root(&(e->obj));
  return;
}

void foci_coeff(int c, const foci_term *arg, foci_term *e){
  e->obj = callback2(*closure_foci_coeff, Val_int(c), arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_sum(int nargs, const foci_term *args, foci_term *e){
  e->obj = callback(*closure_foci_sum, list_of_expr_array(nargs,args));
  register_global_root(&(e->obj));
  return;
}

void foci_equality(const foci_term *arg1, const foci_term *arg2, foci_pred *e){
  e->obj = callback2(*closure_foci_equality, arg1->obj, arg2->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_leq(const foci_term *arg1, const foci_term *arg2, foci_pred *e){
  e->obj = callback2(*closure_foci_leq, arg1->obj, arg2->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_and(int nargs, const foci_pred *args, foci_pred *e){
  e->obj = callback(*closure_foci_and, list_of_expr_array(nargs,(foci_term *)args));
  register_global_root(&(e->obj));
  return;
}

void foci_or(int nargs, const foci_pred *args, foci_pred *e){
  e->obj = callback(*closure_foci_or, list_of_expr_array(nargs,(foci_term *)args));
  register_global_root(&(e->obj));
  return;
}

void foci_not(const foci_pred *arg, foci_pred *e){
  e->obj = callback(*closure_foci_not, arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_ite(const foci_pred *arg1, const foci_term *arg2, const foci_term *arg3, foci_term *e){
  e->obj = callback3(*closure_foci_ite, arg1->obj, arg2->obj, arg3->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_term_dup(const foci_term *arg, foci_term *e){
  e->obj = arg->obj;
  if(arg->obj)register_global_root(&(e->obj));
  return;
}

void foci_term_destroy(foci_term *arg){
  if(arg->obj)remove_global_root(&(arg->obj));
  return;
}

void foci_pred_dup(const foci_pred *arg, foci_pred *e){
  e->obj = arg->obj;
  if(arg->obj)register_global_root(&(e->obj));
  return;
}

void foci_pred_destroy(foci_pred *arg){
  if(arg->obj)remove_global_root(&(arg->obj));
  return;
}

foci_type foci_term_type(const foci_term *arg){
  return Int_val(callback(*closure_foci_term_type,arg->obj));
}

foci_type foci_pred_type(const foci_pred *arg){
  return Int_val(callback(*closure_foci_pred_type,arg->obj));
}

int foci_constant_value(const foci_term *arg){
  return Int_val(callback(*closure_foci_constant_value,arg->obj));
}

const char *foci_atom_name(const foci_pred *arg){
  return String_val(callback(*closure_foci_atom_name,arg->obj));
}

const char *foci_variable_name(const foci_term *arg){
  return String_val(callback(*closure_foci_variable_name,arg->obj));
}

const char *foci_application_name(const foci_term *arg){
  return String_val(callback(*closure_foci_application_name,arg->obj));
}

void foci_application_args(const foci_term *arg, foci_term_list *l){
  l->obj = callback(*closure_foci_application_args,arg->obj);
  register_global_root(&(l->obj));
  return;
}

int foci_coeff_value(const foci_term *arg){
  return Int_val(callback(*closure_foci_coeff_value,arg->obj));
}

void foci_coeff_arg(const foci_term *arg, foci_term *e){
  e->obj = callback(*closure_foci_coeff_arg,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_sum_args(const foci_term *arg, foci_term_list *l){
  l->obj = callback(*closure_foci_sum_args,arg->obj);
  register_global_root(&(l->obj));
  return;
}

void foci_equality_left(const foci_pred *arg, foci_term *e){
  e->obj = callback(*closure_foci_equality_left,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_equality_right(const foci_pred *arg, foci_term *e){
  e->obj = callback(*closure_foci_equality_right,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_leq_left(const foci_pred *arg, foci_term *e){
  e->obj = callback(*closure_foci_leq_left,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_leq_right(const foci_pred *arg, foci_term *e){
  e->obj = callback(*closure_foci_leq_right,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_and_args(const foci_pred *arg, foci_pred_list *l){
  l->obj = callback(*closure_foci_and_args,arg->obj);
  register_global_root(&(l->obj));
  return;
}

void foci_or_args(const foci_pred *arg, foci_pred_list *l){
  l->obj = callback(*closure_foci_or_args,arg->obj);
  register_global_root(&(l->obj));
  return;
}

void foci_not_arg(const foci_pred *arg, foci_pred *e){
  e->obj = callback(*closure_foci_not_arg,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_ite_if(const foci_term *arg, foci_pred *e){
  e->obj = callback(*closure_foci_ite_if,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_ite_then(const foci_term *arg, foci_term *e){
  e->obj = callback(*closure_foci_ite_then,arg->obj);
  register_global_root(&(e->obj));
  return;
}

void foci_ite_else(const foci_term *arg, foci_term *e){
  e->obj = callback(*closure_foci_ite_else,arg->obj);
  register_global_root(&(e->obj));
  return;
}

int foci_interpolate(int nargs, const foci_pred *args, foci_pred *interps, int const_limit, foci_pred_list *quants){
  CAMLparam0();
  CAMLlocal1(res);
  res = callback3(*closure_foci_interpolate, list_of_expr_array(nargs,(foci_term *)args), Val_int(const_limit), Val_bool(quants));
  if(!Bool_val(Field(res,0))) CAMLreturn(0);
  list_to_expr_array(Field(res,1),nargs-1,interps);
  if(quants){
    quants->obj = Field(res,2);
    register_global_root(&(quants->obj));
  }
  CAMLreturn(1);
}

void foci_term_list_null(foci_term_list *e){
  e->obj = 0;
}

void foci_pred_list_null(foci_pred_list *e){
  e->obj = 0;
}

int foci_term_list_empty(const foci_term_list *term_list){
  return !Is_block(term_list->obj);
}

void foci_term_list_first(const foci_term_list *term_list, foci_term *e){
  e->obj = Field(term_list->obj,0);
  register_global_root(&(e->obj));
  return;
}

void foci_term_list_next(foci_term_list *term_list){
  term_list->obj = Field(term_list->obj,1);
}

void foci_term_list_destroy(foci_term_list *arg){
  if(arg->obj)remove_global_root(&(arg->obj));
  return;
}

int foci_pred_list_empty(const foci_pred_list *pred_list){
  return !Is_block(pred_list->obj);
}

void foci_pred_list_first(const foci_pred_list *pred_list, foci_pred *e){
  e->obj = Field(pred_list->obj,0);
  register_global_root(&(e->obj));
  return;
}

void foci_pred_list_next(foci_pred_list *pred_list){
  pred_list->obj = Field(pred_list->obj,1);
}

void foci_pred_list_destroy(foci_pred_list *arg){
  if(arg->obj)remove_global_root(&(arg->obj));
  return;
}

void foci_term_print(const foci_term *arg){
  callback(*closure_foci_term_print,arg->obj);
}

void foci_pred_print(const foci_pred *arg){
  callback(*closure_foci_pred_print,arg->obj);
}

void foci_term_list_dup(const foci_term_list *arg, foci_term_list *e){
  e->obj = arg->obj;
  if(arg->obj)register_global_root(&(e->obj));
  return;
}

void foci_pred_list_dup(const foci_pred_list *arg, foci_pred_list *e){
  e->obj = arg->obj;
  if(arg->obj)register_global_root(&(e->obj));
  return;
}

void foci_print_stats(){
  callback(*closure_foci_print_stats,Val_unit);
}

void foci_gc(){
  callback(*closure_foci_gc,Val_unit);
}
