

#ifndef FOCI_C_INTF_INTERNAL
typedef struct foci_term {void *obj;} foci_term;
typedef struct foci_term_list {void *obj;} foci_term_list;

typedef struct foci_pred {void *obj;} foci_pred;
typedef struct foci_pred_list {void *obj;} foci_pred_list;
#endif

typedef enum {
  foci_type_constant,
  foci_type_variable,
  foci_type_application,
  foci_type_coeff,
  foci_type_sum,
  foci_type_equality,
  foci_type_leq,
  foci_type_and,
  foci_type_or,
  foci_type_not,
  foci_type_ite,
  foci_type_true,
  foci_type_false,
  foci_type_atom
} foci_type;

void foci_c_intf_init();

/* constructors */

void foci_term_null(foci_term *e);
void foci_pred_null(foci_pred *e);
void foci_constant(int c, foci_term *e);
void foci_atom(const char *name, foci_pred *e);
void foci_variable(const char *name, foci_term *e);
void foci_application(const char *name, int nargs, const foci_term *args, foci_term *e);
void foci_coeff(int c, const foci_term*arg, foci_term *e);
void foci_sum(int nargs, const foci_term *args, foci_term *e);
void foci_equality(const foci_term *arg1, const foci_term *arg2, foci_pred *e);
void foci_leq(const foci_term *arg1, const foci_term *arg2, foci_pred *e);
void foci_and(int nargs, const foci_pred *args, foci_pred *e);
void foci_or(int nargs, const foci_pred *args, foci_pred *e);
void foci_not(const foci_pred *arg, foci_pred *e);
void foci_ite(const foci_pred *arg1, const foci_term *arg2, const foci_term *arg3, foci_term *e);
void foci_term_dup(const foci_term *arg, foci_term *e);
void foci_term_destroy(foci_term *arg);
void foci_pred_dup(const foci_pred *arg, foci_pred *e);
void foci_pred_destroy(foci_pred *arg);

/* destructors */

foci_type foci_term_type(const foci_term *arg);
foci_type foci_pred_type(const foci_pred *arg);

int foci_constant_value(const foci_term *arg);
const char *foci_atom_name(const foci_pred *arg);
const char *foci_variable_name(const foci_term *arg);
const char *foci_application_name(const foci_term *arg);
void foci_application_args(const foci_term *arg, foci_term_list *l);
int foci_coeff_value(const foci_term *arg);
void foci_coeff_arg(const foci_term *arg, foci_term *e);
void foci_sum_args(const foci_term *arg, foci_term_list *l);
void foci_equality_left(const foci_pred *arg, foci_term *e);
void foci_equality_right(const foci_pred *arg, foci_term *e);
void foci_leq_left(const foci_pred *arg, foci_term *e);
void foci_leq_right(const foci_pred *arg, foci_term *e);
void foci_and_args(const foci_pred *arg, foci_pred_list *l);
void foci_or_args(const foci_pred *arg, foci_pred_list *l);
void foci_not_arg(const foci_pred *arg, foci_pred *e);
void foci_ite_if(const foci_term *arg, foci_pred *e);
void foci_ite_then(const foci_term *arg, foci_term *e);
void foci_ite_else(const foci_term *arg, foci_term *e);

/* the main event */

int foci_interpolate(int nargs, const foci_pred *args, foci_pred *interps, int const_limit, foci_pred_list *quants);

/* list manipulation */

void foci_term_list_null(foci_term_list *e);
void foci_pred_list_null(foci_pred_list *e);

int foci_term_list_empty(const foci_term_list *term_list);
void foci_term_list_first(const foci_term_list *term_list, foci_term *e);
void foci_term_list_next(foci_term_list *term_list);
void foci_term_list_dup(const foci_term_list *arg, foci_term_list *e);
void foci_term_list_destroy(foci_term_list *arg);

int foci_pred_list_empty(const foci_pred_list *expr_list);
void foci_pred_list_first(const foci_pred_list *expr_list, foci_pred *e);
void foci_pred_list_next(foci_pred_list *expr_list);
void foci_pred_list_dup(const foci_pred_list *arg, foci_pred_list *e);
void foci_pred_list_destroy(foci_pred_list *arg);

/* misc */

void foci_term_print(const foci_term *term);
void foci_pred_print(const foci_pred *pred);
void foci_print_stats();
void foci_gc();
