
extern "C" {
#include "foci_c_intf.h"
}

#include <vector>

struct Foci_term {
  foci_term t;
  Foci_term(){foci_term_null(&t);}
  Foci_term(const Foci_term &q){foci_term_dup(&q.t,&t);}
  Foci_term &operator=(const Foci_term &q){
    foci_term_destroy(&t);
    foci_term_dup(&q.t,&t);
    return *this;
  }
  ~Foci_term(){foci_term_destroy(&t);}
};

struct Foci_pred {
  foci_pred t;
  Foci_pred(){foci_pred_null(&t);}
  Foci_pred(const Foci_pred &q){foci_pred_dup(&q.t,&t);}
  Foci_pred &operator=(const Foci_pred &q){
    foci_pred_destroy(&t);
    foci_pred_dup(&q.t,&t);
    return *this;
  }
  ~Foci_pred(){foci_pred_destroy(&t);}
};

struct Foci_term_list {
  foci_term_list t;
  Foci_term_list(){foci_term_list_null(&t);}
  Foci_term_list(const Foci_term_list &q){foci_term_list_dup(&q.t,&t);}
  Foci_term_list &operator=(const Foci_term_list &q){
    foci_term_list_destroy(&t);
    foci_term_list_dup(&q.t,&t);
    return *this;
  }
  ~Foci_term_list(){foci_term_list_destroy(&t);}
};

struct Foci_pred_list {
  foci_pred_list t;
  Foci_pred_list(){foci_pred_list_null(&t);}
  Foci_pred_list(const Foci_pred_list &q){foci_pred_list_dup(&q.t,&t);}
  Foci_pred_list &operator=(const Foci_pred_list &q){
    foci_pred_list_destroy(&t);
    foci_pred_list_dup(&q.t,&t);
    return *this;
  }
  ~Foci_pred_list(){foci_pred_list_destroy(&t);}
};

typedef foci_type Foci_type;

/* constructors */

void Foci_constant(int c, Foci_term &e);
void Foci_atom(const char *name, Foci_pred &e);
void Foci_variable(const char *name, Foci_term &e);
void Foci_application(const char *name, const std::vector<Foci_term> &args, Foci_term &e);
void Foci_coeff(int c, const Foci_term &arg, Foci_term &e);
void Foci_sum(const std::vector<Foci_term> &args, Foci_term &e);
void Foci_equality(const Foci_term &arg1, const Foci_term &arg2, Foci_pred &e);
void Foci_leq(const Foci_term &arg1, const Foci_term &arg2, Foci_pred &e);
void Foci_and(const std::vector<Foci_pred> &args, Foci_pred &e);
void Foci_or(const std::vector<Foci_pred> &args, Foci_pred &e);
void Foci_not(const Foci_pred &arg, Foci_pred &e);
void Foci_ite(const Foci_pred &arg1, const Foci_term &arg2, const Foci_term &arg3, Foci_term &e);

/* destructors */

Foci_type Foci_term_type(const Foci_term &arg);
Foci_type Foci_pred_type(const Foci_pred &arg);

int Foci_constant_value(const Foci_term &arg);
const char *Foci_atom_name(const Foci_pred &arg);
const char *Foci_variable_name(const Foci_term &arg);
const char *Foci_application_name(const Foci_term &arg);
void Foci_application_args(const Foci_term &arg, Foci_term_list &l);
int Foci_coeff_value(const Foci_term &arg);
void Foci_coeff_arg(const Foci_term &arg, Foci_term &e);
void Foci_sum_args(const Foci_term &arg, Foci_term_list &l);
void Foci_equality_left(const Foci_pred &arg, Foci_term &e);
void Foci_equality_right(const Foci_pred &arg, Foci_term &e);
void Foci_leq_left(const Foci_pred &arg, Foci_term &e);
void Foci_leq_right(const Foci_pred &arg, Foci_term &e);
void Foci_and_args(const Foci_pred &arg, Foci_pred_list &l);
void Foci_or_args(const Foci_pred &arg, Foci_pred_list &l);
void Foci_not_arg(const Foci_pred &arg, Foci_pred &e);
void Foci_ite_if(const Foci_term &arg, Foci_pred &e);
void Foci_ite_then(const Foci_term &arg, Foci_term &e);
void Foci_ite_else(const Foci_term &arg, Foci_term &e);

/* the main event */

int Foci_interpolate(const std::vector<Foci_pred> &args, std::vector<Foci_pred> &interps, int const_limit, Foci_pred_list *quants);

/* list manipulation */

bool Foci_term_list_empty(const Foci_term_list &term_list);
void Foci_term_list_first(const Foci_term_list &term_list, Foci_term &e);
void Foci_term_list_next(Foci_term_list &term_list);

bool Foci_pred_list_empty(const Foci_pred_list &expr_list);
void Foci_pred_list_first(const Foci_pred_list &expr_list, Foci_pred &e);
void Foci_pred_list_next(Foci_pred_list &expr_list);

/* misc */

void Foci_term_print(const Foci_term &term);
void Foci_pred_print(const Foci_pred &pred);
void Foci_print_stats();
void Foci_gc();

/* some overloads for convenience */

Foci_pred operator&(const Foci_pred &p, const Foci_pred &q);
Foci_pred operator|(const Foci_pred &p, const Foci_pred &q);
inline Foci_pred operator~(const Foci_pred &p){Foci_pred r; Foci_not(p,r); return r;}
inline Foci_pred eq(const Foci_term &p, const Foci_term &q){Foci_pred r; Foci_equality(p,q,r); return r;}
inline Foci_pred operator<=(const Foci_term &p, const Foci_term &q){Foci_pred r; Foci_equality(p,q,r); return r;}
inline Foci_pred Foci_true(){std::vector<Foci_pred> args; Foci_pred r; Foci_and(args,r); return r;}
inline Foci_pred Foci_false(){std::vector<Foci_pred> args; Foci_pred r; Foci_or(args,r); return r;}
inline Foci_term Foci_constant(int c){Foci_term r; Foci_constant(c,r); return r;}
inline Foci_pred Foci_atom(const char *name){Foci_pred r; Foci_atom(name,r); return r;}
inline Foci_term Foci_variable(const char *name){Foci_term r; Foci_variable(name,r); return r;}
inline Foci_term Foci_application(const char *name, const std::vector<Foci_term> args){Foci_term r; Foci_application(name,args,r); return r;}


