/* A simplified memory allocator based on the description in
     Scalable lock-free dynamic memory allocation
     M. Michael
     PLDI 2004

   We make the following simplifying assumptions:
   1. Memory is modeled as (type-safe) arrays, so pointers are integer indices
      into array.
   2. Bit-packed fields are expanded as structures.
   3. Initially, we assume atomic operations of the allocation/free routines
   4. There is just one sizeclass
   5. there are no requests for "big" blocks that get sent to the OS 

   Author: Rupak Majumdar rupak@cs.ucla.edu
   v0.1 05/22/2008
*/

#define MEMSIZE 2
#define BLOCKS 2
#define BLOCKSIZE 2

#define NULL 0
// states:
#define ACTIVE 	0
#define FULL   	1
#define PARTIAL	2
#define EMPTY	3

// blast related
#define GHOST

#ifdef __RUN__
int choose() {
	return (rand() % MEMSIZE);
}
#else
int choose() ; // nondeterministic choice
#endif

void assert(int p) {
	if (p == 0) {
#ifdef __RUN__
		abort();
#else
ERROR:		goto ERROR;
#endif
	}
}

/************************************************************/
// memory model

// descriptor : book-keeping for a superblock
typedef struct _desc {
	int avail; // head of available list
} descriptor;

//block : data + ptr to superblock's descriptor
typedef struct _block {
	struct _superblock *sb;
	int data[BLOCKSIZE];
	GHOST int allocated;
} block;

//superblock : a descriptor and an array of blocks
typedef struct _superblock {
	descriptor desc;
	block	   blocks[BLOCKS];
} superblock; 

//heap : an array of superblock
superblock	Heap[MEMSIZE];

void *my_malloc(unsigned int sz, int i) {
	int b;
	superblock *s ;

	s = & Heap[i];
	if (s->desc.avail == -1) return NULL; // no blocks available
	b = s->desc.avail;
foo();
	s->desc.avail = s->blocks[b].data[0];	
foo();
	s->blocks[b].sb = s;
foo();
	s->blocks[b].allocated = 1;
assert(0); // BAD
foo();
	assert(s->blocks[b].allocated == 1);
	return (void *)& s->blocks[b];
}

// push ptr back to free list of superblock
void my_free(block * ptr) {
	superblock *sb ;
	assert(0);
	if (!ptr) return;

	sb =  ptr->sb;
	ptr->data[0] = sb->desc.avail;
	ptr->allocated = 0;
	sb->desc.avail = (ptr - sb->blocks) ; // sizeof(block);
	return;	
}

init() {
	int i, j;
	superblock *sb;
	for (i = 0; i < MEMSIZE; i++) {
		// initialize superblock
		sb = & Heap[i];	
		sb->desc.avail = 0;
		for (j = 0 ; j < BLOCKS - 1; j++) {
			sb->blocks[j].sb = sb;
			sb->blocks[j].allocated = 0;
			sb->blocks[j].data[0] = j+1;	
		}
		sb->blocks[BLOCKS-1].sb = sb;
		sb->blocks[BLOCKS-1].allocated = 0;
		sb->blocks[BLOCKS-1].data[0] = -1;
		{__blockattribute__((assume("forall __BLAST_q_j sb@init->blocks[__BLAST_q_j].allocated == 0;"))) }
	}
	{__blockattribute__((assume("forall __BLAST_q_i forall __BLAST_q_j Heap[__BLAST_q_i].blocks[__BLAST_q_j].allocated == 0;"))) }
	{__blockattribute__((assume("forall __BLAST_q_i forall __BLAST_q_j ((REACH(Heap[__BLAST_q_i].desc.avail, __BLAST_q_j, __BLAST_q_j)==0) ||(Heap[__BLAST_q_i].blocks[__BLAST_q_j].allocated == 0));"))) }
}

#define CLIENTSZ 2
block *clients[CLIENTSZ];
void client_init() {
	int i;
	for (i = 0; i < CLIENTSZ; i++) {
		clients[CLIENTSZ] = NULL;
	}	
	{__blockattribute__((assume("forall __BLAST_q_i (clients[__BLAST_q_i] == 0);"))) }
}

void client_alloc(j) {
	int i = choose();
	block *p = (block *)my_malloc(32, i);
	clients[j] = p;
}
int main() {
	unsigned int __BLAST_NONDET;
	block *p;
//	init();
//	client_init();
//	{__blockattribute__((assume("forall __BLAST_q_i forall __BLAST_q_j Heap[__BLAST_q_i].blocks[__BLAST_q_j].allocated == 0;"))) }
	{__blockattribute__((assume("Heap[0].blocks[0].allocated == 0;")))}
	{__blockattribute__((assume("Heap[0].blocks[1].allocated == 0;")))}
	{__blockattribute__((assume("Heap[0].desc.avail == 0;")))}
	{__blockattribute__((assume("Heap[0].blocks[0].data[0] == 1;")))}
	{__blockattribute__((assume("Heap[0].blocks[1].data[0] == -1;")))}
/*
	while(__BLAST_NONDET) {
		int client = choose();
		if(clients[client]!= 0) {
			assert(clients[client]->allocated == 1);
			my_free(clients[client]);
			clients[client] = 0;
		} else {
			client_alloc(client);
		} 
	}
*/
	p = (block *)my_malloc(32,0);
	if (p!= 0) {
	  assert(p->allocated == 1);
	  my_free(p);
        }
	return 0;
}
