(* $Id: list_util.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Utility functions for lists. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open List

let count p l = 
  let count_one sum x = if p x then sum + 1 else sum in
  let total = fold_left count_one 0 l in
    assert (total >= 0);
    total

let exists_unique p l = 
  (count p l = 1)

let rec rev_rem_dup comp x acc = function
  | [] -> acc
  | (h::t) -> 
      if comp h x = 0 then 
	rev_rem_dup comp x acc t 
      else 
	rev_rem_dup comp h (h::acc) t

let remove_sorted_duplicates comp l =
  match List.sort (Function.flip comp) l with
    | [] -> l
    | (h::t) -> rev_rem_dup comp h [h] t

module Hashed_list (H: Hashtbl.HashedType) =
struct
  type t = H.t list

  let prime = 23
  let depth = 8
    
  let rec hash_limited acc limit elts =
    match (limit,elts) with 
      | (0,_) | (_,[]) -> acc
      | (_,h::t) -> hash_limited (prime * acc + H.hash h) (limit - 1) t

  let hash = hash_limited 0 depth

  let rec equal e f = 
    match (e,f) with
      | ([],[]) -> true
      | ([],_) | (_,[]) -> false
      | (h1::t1,h2::t2) -> H.equal h1 h2 && equal t1 t2
end
