(* $Id: list_util.mli 1793 2007-09-28 19:12:36Z jgrundy $ *)
(** Utility functions for lists. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(* A module of hashable lists built element-wise from a hashable
   element type. *)
module Hashed_list (H: Hashtbl.HashedType) : Hashtbl.HashedType 
  with type t = H.t list

val count: ('a -> bool) -> 'a list -> int
  (** [count p l] is the number of elements in [l] for which [p] holds.

      - Ensure: [count p l >= 0] *)

val exists_unique: ('a -> bool) -> 'a list -> bool
  (** [exists_unique p l] holds when [p] for exactly one element of [l]. 

      - Ensure: [exists_unique p l = (count p l = 1)] *)

val remove_sorted_duplicates: ('a -> 'a -> int) -> 'a list -> 'a list
  (** A list sorted in ascending order according to a comparison
      function, with duplicate elements removed.  The comparison
      function returns a positive integer if the first argument is
      considered greater than the second, and a negative integer if it
      is considered less. If the comparison function returns zero for
      two elements then they are considered equal, both for sorting
      and duplicate removal. *)
