(* $Id: util.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Utility modules and functions. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

module type ComparedType = sig
  type t
  val equal: t -> t -> bool
  val hash: t -> int
  val compare: t -> t -> int
end

module Integer = struct
  type t = int
  let equal x y = x = y
  let hash x = x
  let compare = compare
end

module String = struct
  type t = string
  let equal x y = x = y
  let hash = Hashtbl.hash
  let compare = compare

  (* (pre s0 n0 l0 s1 n1 l1) holds iff 
     string s0[n0..l0) is a prefix of s1[n1..l1) *)
  let rec pre s0 n0 l0 s1 n1 l1 =
    n0 = l0 || 
    (n1 != l1 && s0.[n0] = s1.[n1] && pre s0 (n0 + 1) l0 s1 (n1 + 1) l1) 
    
  (* (prefix s0 s1) holds iff string s0 is a prefix of s1 *)
  let prefix s0 s1 = pre s0 0 (String.length s0) s1 0 (String.length s1)
end

