(* $Id: smtlib_parser.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Combined SMT-LIB file parser interface. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

type 'a tokenizer = Lexing.lexbuf  -> 'a

type ('a,'b) recognizer =  'a tokenizer ->  Lexing.lexbuf -> 'b

let state = ref None

let initialized () = Option.is_some (!state)

let dir () = let (d, _, _, _) = Option.value !state in d

let theory_parser () = let (_, t, _, _) = Option.value !state in t

let logic_parser () = let (_, _, l, _) = Option.value !state in l

let benchmark_parser () = let (_, _, _, b) = Option.value !state in b

let initialize dir tokenizer theory logic benchmark =
  state := Some (dir, theory tokenizer, logic tokenizer, benchmark tokenizer);
  assert (initialized ())

let smtlib_theory_file theory = 
  assert (initialized ());
  let theory_dir = Filename.concat (dir ()) "theories" in
    Filename.concat theory_dir (theory ^ ".smt")
    
let smtlib_logic_file logic = 
  assert (initialized ());
  let logic_dir = Filename.concat (dir ()) "logics" in
    Filename.concat logic_dir (logic ^ ".smt")

let parse start file =
  let f = open_in file in
  let lexbuf = Lexing.from_channel f in
  let result = start lexbuf in
    close_in f;
    result
      
let parse_theory file = 
  assert (initialized ());
  parse (theory_parser ()) file

let parse_logic file = 
  assert (initialized ());
  parse (logic_parser ()) file

let parse_benchmark file = 
  assert (initialized ());
  parse (benchmark_parser ()) file
