(* $Id: smtlib_parser.mli 1788 2007-09-28 18:23:28Z jgrundy $ *)
(** Combined SMT-LIB file parser interface. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** A parser for the SMT-LIB syntax as described in
    {{:http://combination.cs.uiowa.edu/smtlib/papers.html} The SMT-LIB
    Standard} Version 1.2 (30 August 2006). *)

(** Generic tokenizer. *)
type 'a tokenizer = Lexing.lexbuf  -> 'a

(** Generic parser. *)
type ('a,'b) recognizer = 'a tokenizer ->  Lexing.lexbuf -> 'b

val initialized: unit -> bool
  (** Has the parser been initialized? *)

val initialize: 
  string -> 'a tokenizer -> 
  ('a, Smtlib_attribute.attribute list) recognizer ->
  ('a, Smtlib_attribute.attribute list) recognizer ->
  ('a, Smtlib_attribute.attribute list) recognizer ->
  unit
    (** [initialize smtlib tokenizer theory_parser logic_parser
	benchmark_parser] initializes the combined smtlib file parser.
	
	- [smtlib] is the path to the standard SMT-LIB library files
	- [tokenizer] is the tokenizer to use
	- [theory_parser] is the entry point for the theory nonterminal
	- [logic_parser] is the entry point for the logic nonterminal
	- [benchmark_parser] is the entry point for the benchmark nonterminal

	- Ensure [initialized]
    *)

val smtlib_theory_file: string -> string
  (** [smtlib_theory_file name] is the path to the SMT-LIB theory file
      describing the theory called [name].
      
      - Require [initialized] *)

val smtlib_logic_file: string -> string
  (** [smtlib_logic_file name] is the path to the SMT-LIB logic file
      describing the logic called [name].

      - Require [initialized] *)
  
val parse_theory: string -> Smtlib_attribute.attribute list
  (** [parse_theory path] parses the SMT-LIB theory file at the
      specified path and returns the attributes of theory. 

      - Require [initialized] *)

val parse_logic: string -> Smtlib_attribute.attribute list
  (** [parse_logic path] parses the SMT-LIB logic file at the
      specified path and returns the attributes of logic.

      - Require [initialized] *)

val parse_benchmark: string -> Smtlib_attribute.attribute list
  (** [parse_logic path] parses the SMT-LIB benchmark file at the
      specified path and returns the attributes of benchmark.

      - Require [initialized] *)
