(* $Id: smtlib_syntax.mli 1793 2007-09-28 19:12:36Z jgrundy $ *)
(** Compound syntactic constructs occurring in SMT-LIB terms and formulas. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** The current setup for SMT-LIB input makes the following
    assumptions:
    
    - the benchmark may contain the sort Int or Real,
    but not both *)

open Term

exception Syntax_error
  (** Raised when request is not possible for valid SMT-LIB syntax. *)

(** {2 Declaration of the Constants and Predicates} *)

val declare_constant: string -> string list -> Constant.t
  (** [declare_constant c] declares a function symbol.
      
      - Require [declare_constant] has not been called before with [c] *)
  
val declare_predicate: string -> string list -> Constant.t
  (** [declare_predicate p] declares a predicate symbol.

      - Require [declare_predicate] has not been called before with [p] *)

(** {2 Constant and Variable Creation} *)

val integer: string -> Constant.t
  (** [integer i] is an integer constant with value i.

      - Require i is convertible to an int *)

val rational: string -> Constant.t
  (** [rational r] creates a rational constant with value r.
      
      - Require [r] is convertible to a rational *)

val constant: string -> Constant.t
  (** A symbolic constant. *)


val variable: string -> Variable.t
  (** A variable. *)

(** {2 Term Creation} *)

val application: string -> term list -> term
  (** Application of a named function to a list of arguments. *)

val predicate: string -> term list -> term
  (** Application of a named predicate to a list of arguments. *)

val connection: string -> term list -> term
  (** Application of a named logical connective to a list of arguments. *)

val quantification: string -> Variable.t list -> term -> term
  (** A named quantification over a list of variables. *)

val local_let: Variable.t -> term -> term -> term
  (** Local binding of an expression variable. *)

val local_flet: Variable.t -> term -> term -> term
  (** Local binding of a formula variable. *)
