(* $Id: cc_literal.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Literal representation for Congruence Closure. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

module Variable = Id.Nat_generator

(* A literal is an equality or a disequality between two variables *)
type literal = 
  | Equality of Variable.t * Variable.t
  | Disequality of Variable.t * Variable.t

let invalid_literal = Equality (Variable.invalid, Variable.invalid)

let is_valid_literal = function
  | Equality (a,b) | Disequality (a,b) -> 
      Variable.is_valid a && Variable.is_valid b

let equal l m = 
  assert (is_valid_literal l);
  assert (is_valid_literal m);
  match (l,m) with
    | (Equality (a,b), Equality (x,y))
    | (Disequality (a,b), Disequality (x,y)) -> 
	(a = x && b = y) || (a = y && b = x)
    | _ -> false

let negation l =
  assert (is_valid_literal l);
  let n = match l with 
    | Equality (a,b) -> Disequality (a,b)
    | Disequality (a,b) -> Equality (a,b)
  in
    assert (is_valid_literal n);
    assert (not (equal l n));
    n

module Variable_map  = Index.Nat_map
module Variable_hash = Hashtbl.Make (Variable)
module Variable_map2 = Index.Nat_map2 (Variable)
module Variable_set  = Index.Nat_set
