(* $Id: solver_api.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Term level interface specifications. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

class type dpll = 
object
  method owns_constant: term -> bool
  method proxy: term -> term
  method variables: Term_set.t
  method add_interface_variable: term -> unit
  method add_clause: term list -> unit

  method decision_level: int
  method decision_possible: bool
  method inference_possible: bool
  method conflict_detected: bool
  method backjump_possible: bool
  method satisfied: bool
  method unsatisfiable: bool

  method is_interface_literal: term -> bool
  method is_true: term -> bool

  method decide: unit
  method infer: unit
  method set_literal: term -> unit
  method get_literal_possible: bool
  method get_literal: term

  method set_conflict_clause: term list -> unit
  method literal_to_explain: term
  method set_literal_explanation: term list -> unit
  method backjump: unit

  method forget_some_learned_clauses: unit
  method restart: unit

  method num_vars: int
  method num_clauses: int
  method num_learneds: int
  method num_conflicts: int
  method num_decisions: int
  method num_inferences: int
  method num_externally_set: int
end

class type theory = 
object
  method owns_constant: term -> bool
  method proxy: term -> term
  method add_interface_variable: term -> unit
  method variables: Term_set.t

  method decision_level: int
  method inference_possible: bool
  method conflict_detected: bool

  method increment_level: unit
  method infer: unit
  method is_interface_literal: term -> bool
  method is_true: term -> bool
  method set_literal: term -> unit
  method get_literal_possible: bool
  method get_literal: term
  method literal_explanation: term -> term list
  method conflict_clause: term list

  method backjump: int -> unit
  method restart: unit
end

class type solver =
object
  method add_clause: term list -> unit
  method solve: bool
end
