(* $Id: solver_dpllt.mli 1788 2007-09-28 18:23:28Z jgrundy $ *)
(** A DPLL(T) solver. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** This module implements a DPLL(T) style scheme for combining a DPLL
    solver with a theory solver.  The combination scheme is described
    in the following papers.

    - Harald Ganzinger, George Hagen, Robert Nieuwenhuis, Albert
    Oliveras, Cesare Tinelli.  {{:http://dx.doi.org/10.1007/b98490}
    DPLL({i T}): Fast decision procedures}.  {i Lecture Notes in
    Computer Science} 3114:175-188, 2004.
    - Robert Nieuwenhuis, Albert Oliveras, Cesare Tinelli.
    {{:http://doi.acm.org/10.1145/1217856.1217859} Solving SAT and
    SAT modulo theories: From an abstract
    Davis-Putnam-Logemann-Loveland procedure to DPLL({i T})}.  {i
    Journal of the ACM} 53(6):937-977, 2006.
    - Sava Krsic, Amit Goel.
    {{:http://dx.doi.org/10.1007/978-3-540-74621-8_1} Architecting
    Solvers for SAT Modulo Theories: Nelson-Oppen with DPLL}.  {i
    Lecture Notes in Computer Science} 4720:1-27, 2007. *)


class make
  : (unit -> Solver_api.dpll) -> (unit -> Solver_api.theory) 
  -> Solver_api.solver

