(* $Id: symbol.ml 1801 2007-09-29 00:22:43Z jgrundy $ *)
(* Symbols. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

type t = {name: string; arity: int; interpreted: bool}

(* Only one symbol may be declared for a given name, so we have
   compare symbols simply by name. *)

let equal x y = x.name = y.name

let hash x = Hashtbl.hash x.name

let compare x y = String.compare x.name y.name

module Name_hashtbl = Hashtbl.Make (Util.String)

let table = Name_hashtbl.create 0

let name_declared name = Name_hashtbl.mem table name

let declare' name arity interpreted =
  assert (name != "");
  assert (arity >= 0);
  assert (not (name_declared name));
  let symbol = {name = name; arity = arity; interpreted = interpreted} in
    Name_hashtbl.add table name symbol;
    assert (name_declared name);
    symbol

let declare name arity =
  declare' name arity true

let declare_uninterpreted name arity =
  declare' name arity false


let find n = Name_hashtbl.find table n

let arity s = s.arity

let interpreted s = s.interpreted

let string_of_symbol s = s.name

let pp_print f s = Format.pp_print_string f s.name

module Primitive = struct

  let truth            = declare "true"         0
  let falsity          = declare "false"        0
  let equality         = declare "="            2
  let negation         = declare "not"          1
  let disjunction      = declare "or"           2
  let conjunction      = declare "and"          2
  let difference       = declare "xor"          2
  let equivalence      = declare "iff"          2
  let implication      = declare "implies"      2
  let conditional      = declare "if_then_else" 3
  let ite              = declare "ite"          3
  let less             = declare "<"            2
  let less_or_equal    = declare "<="           2 
  let greater          = declare ">"            2
  let greater_or_equal = declare ">="           2
  let unary_minus      = declare "~"            1
  let plus             = declare "+"            2
  let minus            = declare "-"            2
  let times            = declare "*"            2
  let divide           = declare "/"            2

end
