(* $Id: term_constant.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Predefined Constants. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

(* Boolean *)
let falsity          = symbolic_constant Symbol.Primitive.falsity
let truth            = symbolic_constant Symbol.Primitive.truth
let negation         = symbolic_constant Symbol.Primitive.negation
let conjunction      = symbolic_constant Symbol.Primitive.conjunction
let disjunction      = symbolic_constant Symbol.Primitive.disjunction
let equivalence      = symbolic_constant Symbol.Primitive.equivalence
let difference       = symbolic_constant Symbol.Primitive.difference
let implication      = symbolic_constant Symbol.Primitive.implication
let conditional      = symbolic_constant Symbol.Primitive.conditional

(* Arithmetic *)
let unary_minus      = symbolic_constant Symbol.Primitive.unary_minus
let plus             = symbolic_constant Symbol.Primitive.plus
let minus            = symbolic_constant Symbol.Primitive.minus
let times            = symbolic_constant Symbol.Primitive.times
let divide           = symbolic_constant Symbol.Primitive.divide
let less             = symbolic_constant Symbol.Primitive.less
let less_or_equal    = symbolic_constant Symbol.Primitive.less_or_equal
let greater          = symbolic_constant Symbol.Primitive.greater
let greater_or_equal = symbolic_constant Symbol.Primitive.greater_or_equal

(* Polymorphic *)
let equality         = symbolic_constant Symbol.Primitive.equality
let ite              = symbolic_constant Symbol.Primitive.ite

(* Predicates *)

let boolean_constants = 
  List.fold_right Term_set.add 
    [falsity; truth; negation; conjunction; disjunction; equivalence;
     difference; implication; conditional] Term_set.empty

let logical_constants = 
   List.fold_right Term_set.add [equality; ite] boolean_constants

let is_boolean_constant term =
  Term_set.mem term boolean_constants

let is_logical_constant term = 
  Term_set.mem term logical_constants
