(* $Id: term_constant.mli 1788 2007-09-28 18:23:28Z jgrundy $ *)
(** Predefined Constants. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

(** {2 Boolean Constants} *)

val truth:            term (** Logical true. *)
val falsity:          term (** Logical false. *)
val negation:         term (** Logical negation. *)
val conjunction:      term (** Logical conjunction. *)
val disjunction:      term (** Logical disjunction. *)
val equivalence:      term (** Logical equivalence. *)
val difference:       term (** Exclusive or. *)
val implication:      term (** Logical implication. *)
val conditional:      term (** Logical if-then-else. *)

(** {2 Arithmetic Constants} *)

val unary_minus:      term (** Arithmetic unary minus. *)
val plus:             term (** Arithmetic addition. *)
val minus:            term (** Arithmetic subtraction. *)
val times:            term (** Arithmetic multiplication. *)
val divide:           term (** Arithmetic division. *)
val less:             term (** Less-than relation. *)
val less_or_equal:    term (** Less-than-or-equal relation. *)
val greater:          term (** Greater-than relation. *)
val greater_or_equal: term (** Greater-than-or-equal relation. *)

(** {2 Polymorphic Constants} *)

val equality:         term (** Equality *)
val ite:              term (** If-then-else *)

(** {2 Predicates} *)

val is_boolean_constant: term -> bool
val is_logical_constant: term -> bool
