(* $Id: term_literal.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Term literals. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

let is_literal = function 
  | Variable _ -> true
  | Application (_, f, Variable _) -> term_equal f Term_constant.negation
  | _ -> false
      
let variable l =
  assert (is_literal l);
  let v = 
    match l with
      | Variable _ -> l
      | Application (_, _, negated) -> negated
      | _ -> assert false
  in
    assert (is_variable v);
    v
      
let is_positive l =
  assert (is_literal l);
  match l with
    | Variable _ -> true
    | _ -> false
	
let literal v =
  assert (is_variable v);
  let l = v in
    assert (is_positive l);
    assert (variable l = v);
    l
      
let negation l = 
  assert (is_literal l);
  let l' = 
    match l with
      | Variable _ -> application Term_constant.negation l
      | Application (_, _, negated_term_var) -> negated_term_var
      | _ -> assert false
  in
    assert (is_positive l' = not (is_positive l));
    assert (variable l' = variable l);
    l'


