(* $Id: variable.ml 1788 2007-09-28 18:23:28Z jgrundy $ *)
(* Variables. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

type t = string

let fresh_prefix = "_v_"

let valid_name n = (n != "") && (not (Util.String.prefix fresh_prefix n))

let named n =
  assert (valid_name n);
  n

let generator = Id.Nat_generator.new_generator () 

let fresh () =
  fresh_prefix ^ (string_of_int (Id.Nat_generator.fresh generator))

let rec variant avoid v = 
  if List.mem v avoid then
    variant avoid (v ^ "'")
  else 
    v

let equal v w = (v = w)

let hash v = Hashtbl.hash v

let compare v w = String.compare v w

let string_of v = v

let pp_print formatter v =
  Format.pp_print_string formatter v

let log level v = Log.log pp_print level v
