# $Id: Build.makefile,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $
# Describes individual files to build and how to build them.

# Copyright 2007 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you
# may not use this file except in compliance with the License.  You
# may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.  See the License for the specific language governing
# permissions and limitations under the License.

##############################################################################
# Variables, to be modified by the package build files

MLIFILES    :=  
MLFILES     :=
MLLFILES    := 
MLYFILES    :=
MLEXES      :=
DOCFILES    :=

##############################################################################
# Get configuration and package information

include Config.makefile
include Pkgs.makefile
-include $(LOCALDIR)/Build.makefile

##############################################################################
# Build information

INCLUDES  = $(foreach pkg, $(PKGS), -I $(pkg))
BUILDS    = $(foreach pkg, $(PKGS), $(pkg)/BUILD.makefile)
include $(BUILDS)

GEN_ML    = $(addsuffix .ml,  $(basename $(MLYFILES) $(MLLFILES)))
GEN_MLI   = $(addsuffix .mli, $(basename $(MLYFILES)))

ALLMLFILES  := $(MLFILES) $(GEN_ML)
ALLMLIFILES := $(MLIFILES) $(GEN_MLI)

CMIFILES  = $(addsuffix .cmi, $(basename $(ALLMLIFILES)))
CMOFILES  = $(shell ocamldsort -d $(OCAMLDEP) $(INCLUDES) -byte $(ALLMLFILES))
CMXFILES  = $(shell ocamldsort -d $(OCAMLDEP) $(INCLUDES) -opt $(ALLMLFILES))

CMXOFILES = $(addsuffix .o, $(basename $(CMXFILES)))

BYTELIB   = $(LIBDIR)/byte_lib.cma
OPTLIB    = $(LIBDIR)/opt_lib.cmxa

OPTEXES   = $(addsuffix .opt, $(MLEXES))
BYTEEXES  = $(addsuffix .byte, $(MLEXES))

SRCDOCS   = $(wildcard $(DOCDIR)/src/*.html)
GENDOCS   = $(addprefix $(DOCDIR)/out/,$(notdir $(SRCDOCS)))
PRGINDEX  = $(DOCDIR)/out/ocamldoc_html/index.html
STDLIBDOC = $(DOCDIR)/out/stdlib.dump

RMFILES   = $(BYTELIB) $(OPTLIB) \
            $(CMIFILES) $(CMOFILES) $(CMXFILES) \
            $(GEN_ML) $(GEN_MLI) \
            $(OPTEXES) $(BYTEEXES)\
            $(TOPLEVEL) .depend \
	    $(DOCDIR)/out/ocamldoc_html/* \
	    $(DOCDIR)/out/*.html \
            $(addprefix $(LIBDIR)/, $(notdir $(MLIFILES))) \
            $(addprefix $(DIR)/, $(notdir $(MLIFILES))) \
            $(addprefix $(LIBDIR)/, $(notdir $(CMIFILES))) \
            $(addprefix $(BINDIR)/, $(notdir $(OPTEXES))) \
            $(addprefix $(BINDIR)/, $(notdir $(BYTEEXES))) \
            $(addsuffix .cm*, $(MLEXES)) \
            $(addsuffix .o, $(MLEXES) $(basename $(CMOFILES))) \
            $(addsuffix .a, $(basename $(OPTLIB))) 

##############################################################################
# Trace functions

SHORTPATH=$(subst $(LOCALDIR),local,$(subst $(ROOTDIR)/,,$1))
DESC=$(call SHORTPATH,$(@)): $(1) $(call SHORTPATH,$(firstword $^))

ifdef VERBOSE
  TRACE=echo $(call DESC,$(1))
else
  TRACE=
endif

##############################################################################
# Generic make rules

%.cmx : %.ml
	@$(call TRACE,$(OCAMLOPT))
	@$(OCAMLOPT) $(OCAML_OPTS) $(NATIVE_OPTS) $(INCLUDES) -c -o $@ $<

%.cmo : %.ml
	@$(call TRACE,$(OCAMLC))
	@$(OCAMLC) $(OCAML_OPTS) $(BYTE_OPTS) $(INCLUDES) -c -o $@ $<

%.cmi : %.mli
	@$(call TRACE,$(OCAMLC))
	@$(OCAMLC) $(OCAML_OPTS) $(BYTE_OPTS) $(INCLUDES) -c -o $@ $<

%.ml %.mli : %.mly
	@$(call TRACE,$(OCAMLYACC))
	@$(OCAMLYACC) $<

%.ml : %.mll
	@$(call TRACE,$(OCAMLLEX))
	@$(OCAMLLEX) $<

%.opt : %.ml $(OPTLIB)
	@$(call TRACE,$(OCAMOPT))
	@$(OCAMLOPT) $(OCAML_OPTS) $(NATIVE_OPTS) $(INCLUDES) -o $@	\
		 $(OPTLIBS) $(OPTLIB) $< 

%.byte : %.ml $(BYTELIB)
	@$(call TRACE,$(OCAMLC))
	@$(OCAMLC) $(OCAML_OPTS) $(BYTE_OPTS) $(INCLUDES) -o $@		\
		 $(BYTELIBS) $(BYTELIB) $< 

$(DOCDIR)/out/%.html: $(DOCDIR)/src/%.html
	@cp $< $@

