(* $Id: function.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Common functions and combinators. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

val id: 'a -> 'a
  (** The identity function. *)

val const: 'a -> 'b -> 'a
  (** Constant function. *)

val compose: ('b -> 'c) -> ('a -> 'b) -> ('a -> 'c)
  (** Function composition. *)

val flip: ('a -> 'b -> 'c) -> 'b -> 'a -> 'c
  (** Flips the order of the first two argument. *)

val curry: ('a * 'b -> 'c) -> 'a -> 'b -> 'c
  (** Curries a function. *)

val uncurry: ('a -> 'b -> 'c) -> 'a * 'b -> 'c
  (** Uncurry a function. *)

val until: ('a -> bool) -> ('a -> 'a) -> 'a -> 'a
  (** [until p f x] applies [f] repeatedly, starting with [x],
      until the predicate [p] evaluates to [true].
      
      Note: If [p x] is [true] then [x] is returned. *)

val fix: (('a -> 'b) -> 'a -> 'b) -> 'a -> 'b
  (** Computes the fix-point for the function transformer. *)
