(* $Id: log.ml,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(* Data logging. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Format
open Time

let log_formatter = ref std_formatter
let log_level = ref 0
let at_newline = ref true

let set_log_formatter formatter = log_formatter := formatter

let set_log_level level = 
  if level < 0 then
    raise (Invalid_argument "negative level")
  else
    log_level := level

let log print level x = 
  if level < 0 then 
    raise (Invalid_argument "negative level")
  else if level <= !log_level then begin
    if !at_newline then begin
      fprintf !log_formatter "%.2f" (since epoch);
      pp_print_string !log_formatter "s: ";
      at_newline := false;
    end;
    print !log_formatter x;
  end

let log_int    level i = log pp_print_int    level i
let log_float  level j = log pp_print_float  level j
let log_string level s = log pp_print_string level s
let log_bool   level b = log pp_print_bool   level b
let log_flush  level _ = log pp_print_flush  level ()

let log_time level t = 
  log_float level (since t);
  log_string level "s"

let log_newline level () =
  at_newline := false;
  log pp_print_newline level (); 
  at_newline := true


