(* $Id: option.ml,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(* Functions for the Option type. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

let is_some = function
  | None -> false
  | Some _ -> true

let is_none = function
  | None -> true
  | Some _ -> false

let value = function
  | None -> raise (Invalid_argument "Option.value")
  | Some x -> x

let default d = function
  | None -> d
  | Some x -> x

let classify p x = 
  if p x then 
    Some x 
  else 
    None

let perform c = function
  | None -> ()
  | Some x -> c x

let map f = function
  | None -> None
  | Some x -> Some (f x)

let filter p = function
  | None -> None
  | Some x -> classify p x

let flatten = function
  | Some x -> x
  | None -> None
