(* $Id: ternary.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Three valued booleans. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(* A ternary is really an int, but we don't want people to exploit
 * that.  Unfortunately, if we don't tell the compiler, it can't
 * exploit it either.  If you remove this comment and reveal type to
 * the compiler you will get about a 20% performance jump in SAT. *)

(** Three valued booleans. *)

(** Ternary values
    
    Treat as if ternary was abstract, but if we make it so then the
    compiler generates slower code. *)
type ternary = int
    
val truth: ternary
  (** true value. *)

val unknown: ternary
  (** unknown value. *)

val falsity: ternary
  (** false value. *)

val ternary_of_bool: bool -> ternary
  (** Injection of a regular boolean into a ternary boolean. *)

val bool_of_ternary: ternary -> bool
  (** [bool_of_ternary t] is the boolean corresponding to a known ternary [t]. 

      - Require [t != ]{!Ternary.unknown} *)

val negation: ternary -> ternary
  (** Negation of a ternary. *)

val conjunction: ternary -> ternary -> ternary
  (** Conjunction of ternary values. *)

val disjunction: ternary -> ternary -> ternary
  (** Disjunction of ternary values. *)

