(* $Id: util.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Utility modules and functions. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** Type of modules that may be used as keys for [Map] and [Hashtbl]. *)
module type ComparedType = sig

  (** Abstract type of comparable values. *)
  type t

  val equal: t -> t -> bool
    (** Are two values equal? 

	- Ensure [equal x y = (]{!Util.ComparedType.compare}[ x y = 0)] *)

  val hash: t -> int
    (** Hash code of a value. *)

  val compare: t -> t -> int
    (** An arbitrary order on values.  [compare x y] is [< 0], [= 0],
	or [> 0] when [x] precedes [y], is equal to [y], or follows
	[y] in the order.

	- Ensure [(compare x y = 0) = ]{!Util.ComparedType.equal}[ x y] *)

end

(** Integers in a form suitable for use with [Map] and [Hashtbl]. *)
module Integer: ComparedType with type t = int

(** Strings in a form suitable for use with [Map] and [Hashtbl]. *)
module String: sig
  include ComparedType with type t = string
  val prefix: string -> string -> bool
    (** [prefix s0 s1] holds if [s0] is a prefix of [s1]. *)
end


