(* $Id: smtlib_attribute.ml,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(* SMT-LIB benchmark, logic and theory attributes. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term
open List
open List_util

type status = 
  | Sat 
  | Unsat 
  | Unknown

type attribute = 
  | Name of string
  | Sorts of string list
  | Funs of Constant.t list
  | Preds of Constant.t list
  | Definition of string
  | Axioms of term list
  | Theory of attribute list
  | Language of string
  | Extensions of string
  | Logic of attribute list
  | Assumption of term
  | Formula of term
  | Status of status
  | Notes of string
  | Annotation of (string * (string option))

let is_name = function 
  | (Name _) -> true
  | _ -> false

let name_value = function
  | (Name n) -> n
  | _ -> assert false

let is_sorts = function
  | (Sorts _) -> true
  | _ -> false

let sorts_value = function
  | (Sorts s) -> s
  | _ -> assert false

let is_funs = function
  | (Funs _) -> true
  | _ -> false

let fun_values = function
  | (Funs f) -> f
  | _ -> assert false

let is_preds = function
  | (Preds _) -> true
  | _ -> false

let preds_value = function
  | (Preds p) -> p
  | _ -> assert false

let is_definition = function
  | (Definition _) -> true
  | _ -> false

let definition_value = function
  | (Definition d) -> d
  | _ -> assert false

let is_axioms = function
  | (Axioms _) -> true
  | _ -> false

let axioms_value = function
  | (Axioms a) -> a
  | _ -> assert false

let is_theory = function
  | (Theory _) -> true
  | _ -> false

let theory_value = function
  | (Theory t) -> t
  | _ -> assert false

let is_extensions = function
  | (Extensions _) -> true
  | _ -> false

let extensions_value = function
  | (Extensions e) -> e
  | _ -> assert false

let is_logic = function
  | (Logic _) -> true
  | _ -> false

let logic_value = function
  | (Logic l) -> l
  | _ -> assert false

let is_assumption = function
  | (Assumption _) -> true
  | _ -> false

let assumption_value = function
  | (Assumption a) -> a
  | _ -> assert false

let is_formula = function
  | (Formula _) -> true 
  | _ -> false

let formula_value = function
  | (Formula f) -> f
  | _ -> assert false

let is_status = function
  | (Status _) -> true 
  | _ -> false

let status_value = function
  | (Status s) -> s
  | _ -> assert false

let is_notes = function
  | (Notes _) -> true 
  | _ -> false

let notes_value = function
  | (Notes n) -> n
  | _ -> assert false

let is_annotation = function
  | (Annotation _) -> true 
  | _ -> false

let annotation_value = function
  | (Annotation a) -> a
  | _ -> assert false

let assumptions attrs = 
  map assumption_value (filter is_assumption attrs)

let formula attrs = 
  assert (exists_unique is_formula attrs);
  formula_value (find is_formula attrs)

let status attrs =
  assert (count is_status attrs < 2);
  if (exists is_status attrs) then
    status_value (find is_status attrs)
  else
    Unknown
