(* $Id: smtlib_attribute.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** SMT-LIB benchmark, logic and theory attributes. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

(** Expected status of a benchmark. *)
type status = 
  | Sat    
  | Unsat  
  | Unknown

(** Attributes that may appear in benchmark, logic and theory files. *)
type attribute = 
  | Name of string
  | Sorts of string list
  | Funs of Constant.t list
  | Preds of Constant.t list
  | Definition of string
  | Axioms of term list
  | Theory of attribute list
  | Language of string
  | Extensions of string
  | Logic of attribute list
  | Assumption of term
  | Formula of term
  | Status of status
  | Notes of string
  | Annotation of (string * (string option))

val assumptions: attribute list -> term list
  (** A list of all the assumptions in a collection of attributes. *)

val formula: attribute list -> term
  (** Formula attribute value from a collection of attributes. *)

val status: attribute list -> status
  (** The status attribute of a collection of attributes. *)

