type token =
  | LEFT_PAREN
  | RIGHT_PAREN
  | BENCHMARK
  | FALSE
  | FLET
  | ITE
  | LET
  | LOGIC
  | THEORY
  | TRUE
  | CONNECTIVE of (string)
  | QUANT_SYMB of (string)
  | STRING of (string)
  | USER_VALUE of (string)
  | NUMERAL of (string)
  | RATIONAL of (string)
  | IDENTIFIER of (string)
  | VAR of (string)
  | FVAR of (string)
  | ATTRIBUTE of (string)
  | ARITH_SYMB of (string)
  | STATUS of (Smtlib_attribute.status)
  | ATTRIBUTE_ASSUMPTION
  | ATTRIBUTE_AXIOMS
  | ATTRIBUTE_DEFINITION
  | ATTRIBUTE_EXTENSIONS
  | ATTRIBUTE_FORMULA
  | ATTRIBUTE_FUNS
  | ATTRIBUTE_EXTRAFUNS
  | ATTRIBUTE_EXTRASORTS
  | ATTRIBUTE_EXTRAPREDS
  | ATTRIBUTE_LANGUAGE
  | ATTRIBUTE_LOGIC
  | ATTRIBUTE_NOTES
  | ATTRIBUTE_PREDS
  | ATTRIBUTE_SORTS
  | ATTRIBUTE_STATUS
  | ATTRIBUTE_THEORY

open Parsing;;
# 3 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
  (** Parser for SMT-LIB theory, logic, and benchmark files. *)
  
  (* Copyright 2007 Intel Corporation 
     
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
     You may obtain a copy of the License at
     
     http://www.apache.org/licenses/LICENSE-2.0
     
     Unless required by applicable law or agreed to in writing,
     software distributed under the License is distributed on an "AS
     IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
     express or implied.  See the License for the specific language
     governing permissions and limitations under the License. *)

  (** {2 Parser API } *)

  (** This grammar recognizes SMT-LIB syntax as described in
      {{:http://combination.cs.uiowa.edu/smtlib/papers.html} The
      SMT-LIB Standard} Version 1.2 (30 August 2006). *)

  open Smtlib_attribute
  open Smtlib_syntax

  module Var_hashtbl = Hashtbl.Make (Variable)

  let rec subst tbl term = 
    match term with
      | Term.Variable (_, v) -> 
          begin
            try Var_hashtbl.find tbl v
            with Not_found -> term
          end
      | Term.Constant _ -> term
      | Term.Application (_, t1, t2) ->
          Term.application (subst tbl t1) (subst tbl t2)
      | _ -> (* SMT-LIB format never constructs abstractions. *)
	  assert false

  let bind_one tbl (var, term) = 
    Var_hashtbl.add tbl var (subst tbl term)

  let bind_local (bindings,body) =
    if bindings = [] then 
      body
    else 
      let tbl = Var_hashtbl.create 0 in
        List.iter (bind_one tbl) bindings;
        subst tbl body
# 94 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
let yytransl_const = [|
  257 (* LEFT_PAREN *);
  258 (* RIGHT_PAREN *);
  259 (* BENCHMARK *);
  260 (* FALSE *);
  261 (* FLET *);
  262 (* ITE *);
  263 (* LET *);
  264 (* LOGIC *);
  265 (* THEORY *);
  266 (* TRUE *);
  279 (* ATTRIBUTE_ASSUMPTION *);
  280 (* ATTRIBUTE_AXIOMS *);
  281 (* ATTRIBUTE_DEFINITION *);
  282 (* ATTRIBUTE_EXTENSIONS *);
  283 (* ATTRIBUTE_FORMULA *);
  284 (* ATTRIBUTE_FUNS *);
  285 (* ATTRIBUTE_EXTRAFUNS *);
  286 (* ATTRIBUTE_EXTRASORTS *);
  287 (* ATTRIBUTE_EXTRAPREDS *);
  288 (* ATTRIBUTE_LANGUAGE *);
  289 (* ATTRIBUTE_LOGIC *);
  290 (* ATTRIBUTE_NOTES *);
  291 (* ATTRIBUTE_PREDS *);
  292 (* ATTRIBUTE_SORTS *);
  293 (* ATTRIBUTE_STATUS *);
  294 (* ATTRIBUTE_THEORY *);
    0|]

let yytransl_block = [|
  267 (* CONNECTIVE *);
  268 (* QUANT_SYMB *);
  269 (* STRING *);
  270 (* USER_VALUE *);
  271 (* NUMERAL *);
  272 (* RATIONAL *);
  273 (* IDENTIFIER *);
  274 (* VAR *);
  275 (* FVAR *);
  276 (* ATTRIBUTE *);
  277 (* ARITH_SYMB *);
  278 (* STATUS *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\002\000\002\000\003\000\004\000\004\000\005\000\
\006\000\006\000\007\000\007\000\008\000\008\000\009\000\010\000\
\010\000\011\000\011\000\011\000\011\000\012\000\012\000\013\000\
\014\000\014\000\014\000\014\000\015\000\015\000\015\000\015\000\
\016\000\016\000\016\000\017\000\017\000\018\000\019\000\020\000\
\020\000\021\000\021\000\021\000\021\000\021\000\023\000\023\000\
\023\000\024\000\022\000\025\000\025\000\026\000\027\000\027\000\
\028\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
\030\000\030\000\031\000\032\000\033\000\034\000\034\000\034\000\
\034\000\034\000\035\000\035\000\036\000\037\000\038\000\039\000\
\039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
\040\000\040\000\041\000\042\000\000\000\000\000\000\000"

let yylen = "\002\000\
\001\000\001\000\001\000\001\000\001\000\001\000\002\000\001\000\
\000\000\001\000\001\000\002\000\001\000\002\000\001\000\000\000\
\001\000\001\000\001\000\001\000\001\000\001\000\002\000\001\000\
\001\000\004\000\005\000\007\000\001\000\001\000\001\000\001\000\
\001\000\004\000\005\000\001\000\002\000\001\000\004\000\002\000\
\002\000\001\000\005\000\005\000\009\000\009\000\005\000\005\000\
\005\000\005\000\001\000\001\000\002\000\001\000\001\000\002\000\
\001\000\004\000\004\000\004\000\002\000\004\000\002\000\001\000\
\001\000\002\000\001\000\005\000\001\000\002\000\002\000\002\000\
\002\000\001\000\001\000\002\000\001\000\005\000\001\000\002\000\
\002\000\002\000\002\000\004\000\004\000\004\000\002\000\001\000\
\001\000\002\000\001\000\005\000\002\000\002\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\000\000\000\000\000\000\093\000\000\000\094\000\
\000\000\095\000\000\000\000\000\000\000\051\000\000\000\069\000\
\000\000\079\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\064\000\065\000\000\000\000\000\000\000\000\000\
\000\000\000\000\074\000\075\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\088\000\089\000\
\000\000\000\000\012\000\000\000\061\000\000\000\063\000\000\000\
\000\000\066\000\068\000\072\000\071\000\073\000\070\000\076\000\
\078\000\000\000\030\000\029\000\032\000\031\000\033\000\042\000\
\081\000\082\000\000\000\000\000\000\000\080\000\087\000\083\000\
\090\000\092\000\000\000\000\000\036\000\000\000\052\000\000\000\
\000\000\000\000\055\000\000\000\000\000\005\000\006\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\004\000\000\000\
\000\000\000\000\000\000\000\000\037\000\062\000\000\000\000\000\
\001\000\002\000\000\000\053\000\059\000\003\000\000\000\056\000\
\060\000\007\000\058\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\019\000\020\000\021\000\018\000\025\000\000\000\
\000\000\022\000\013\000\000\000\000\000\085\000\084\000\086\000\
\000\000\000\000\000\000\010\000\000\000\000\000\000\000\017\000\
\000\000\000\000\000\000\041\000\040\000\000\000\000\000\000\000\
\000\000\000\000\023\000\000\000\014\000\034\000\000\000\000\000\
\000\000\000\000\000\000\000\000\043\000\000\000\044\000\000\000\
\000\000\000\000\035\000\048\000\049\000\047\000\050\000\000\000\
\000\000\039\000\000\000\000\000\026\000\000\000\000\000\000\000\
\027\000\000\000\000\000\000\000\046\000\045\000\028\000"

let yydgoto = "\004\000\
\115\000\104\000\095\000\096\000\097\000\149\000\139\000\140\000\
\152\000\153\000\135\000\136\000\137\000\138\000\071\000\072\000\
\083\000\084\000\128\000\129\000\085\000\015\000\087\000\091\000\
\088\000\089\000\092\000\093\000\028\000\029\000\030\000\006\000\
\017\000\036\000\037\000\038\000\008\000\019\000\048\000\049\000\
\050\000\010\000"

let yysindex = "\060\000\
\016\255\038\255\039\255\000\000\000\255\000\000\034\255\000\000\
\042\255\000\000\026\255\031\255\033\255\000\000\149\255\000\000\
\070\255\000\000\120\255\032\255\058\255\059\255\092\255\082\255\
\096\255\097\255\000\000\000\000\149\255\069\255\088\255\104\255\
\105\255\026\255\000\000\000\000\070\255\103\255\090\255\090\255\
\109\255\123\255\125\255\031\255\128\255\122\255\000\000\000\000\
\120\255\129\255\000\000\090\255\000\000\141\255\000\000\147\255\
\135\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\151\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\141\255\135\255\147\255\000\000\000\000\000\000\
\000\000\000\000\090\255\157\255\000\000\117\255\000\000\141\255\
\162\255\254\254\000\000\147\255\163\255\000\000\000\000\135\255\
\164\255\159\255\166\255\090\255\170\255\000\000\000\000\098\255\
\155\255\174\255\177\255\178\255\000\000\000\000\135\255\135\255\
\000\000\000\000\135\255\000\000\000\000\000\000\135\255\000\000\
\000\000\000\000\000\000\167\255\173\255\155\255\176\255\102\255\
\155\255\172\255\000\000\000\000\000\000\000\000\000\000\098\255\
\155\255\000\000\000\000\155\255\179\255\000\000\000\000\000\000\
\155\255\155\255\155\255\000\000\155\255\090\255\098\255\000\000\
\180\255\135\255\118\255\000\000\000\000\190\255\090\255\000\000\
\098\255\155\255\000\000\194\255\000\000\000\000\195\255\196\255\
\197\255\198\255\199\255\200\255\000\000\201\255\000\000\098\255\
\155\255\202\255\000\000\000\000\000\000\000\000\000\000\090\255\
\090\255\000\000\098\255\203\255\000\000\155\255\155\255\155\255\
\000\000\204\255\205\255\206\255\000\000\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\050\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\207\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\208\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\209\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\008\255\000\000\000\000\000\000\000\000\210\255\
\000\000\000\000\000\000\211\255\000\000\000\000\000\000\010\255\
\000\000\000\000\000\000\000\000\000\000\017\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\018\255\000\000\
\000\000\000\000\000\000\000\000\000\000\212\255\000\000\000\000\
\212\255\000\000\000\000\000\000\000\000\000\000\000\000\029\255\
\212\255\000\000\000\000\213\255\000\000\000\000\000\000\000\000\
\212\255\212\255\212\255\000\000\212\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\040\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\212\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\212\255\212\255\212\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\065\000\126\000\171\255\000\000\188\255\000\000\243\255\000\000\
\158\255\132\255\087\000\000\000\057\000\134\255\193\255\000\000\
\000\000\119\000\000\000\092\000\217\255\187\000\134\000\131\000\
\000\000\149\000\000\000\148\000\197\000\000\000\000\000\000\000\
\183\000\191\000\000\000\000\000\000\000\000\000\180\000\000\000\
\000\000\000\000"

let yytablesize = 229
let yytable = "\073\000\
\074\000\027\000\105\000\035\000\158\000\047\000\141\000\107\000\
\011\000\038\000\122\000\008\000\164\000\163\000\118\000\027\000\
\005\000\003\000\103\000\009\000\167\000\168\000\169\000\035\000\
\170\000\145\000\146\000\038\000\172\000\008\000\024\000\003\000\
\003\000\003\000\003\000\047\000\032\000\009\000\007\000\009\000\
\001\000\012\000\014\000\109\000\013\000\051\000\147\000\016\000\
\024\000\018\000\148\000\011\000\188\000\187\000\001\000\001\000\
\001\000\001\000\052\000\021\000\001\000\002\000\003\000\178\000\
\192\000\194\000\195\000\196\000\174\000\011\000\059\000\053\000\
\011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
\011\000\011\000\011\000\011\000\011\000\011\000\011\000\011\000\
\157\000\020\000\066\000\105\000\054\000\067\000\055\000\031\000\
\056\000\057\000\130\000\068\000\060\000\032\000\155\000\033\000\
\065\000\067\000\069\000\034\000\070\000\075\000\171\000\068\000\
\131\000\132\000\133\000\134\000\061\000\062\000\069\000\176\000\
\070\000\067\000\098\000\076\000\099\000\077\000\165\000\068\000\
\100\000\101\000\082\000\111\000\112\000\113\000\102\000\154\000\
\070\000\114\000\103\000\020\000\079\000\086\000\039\000\080\000\
\190\000\191\000\040\000\090\000\041\000\042\000\043\000\094\000\
\044\000\045\000\067\000\098\000\046\000\099\000\110\000\124\000\
\068\000\100\000\101\000\117\000\121\000\123\000\125\000\102\000\
\020\000\070\000\127\000\103\000\021\000\022\000\020\000\142\000\
\023\000\159\000\143\000\144\000\166\000\173\000\024\000\025\000\
\026\000\150\000\131\000\132\000\160\000\134\000\151\000\175\000\
\114\000\154\000\161\000\179\000\180\000\181\000\182\000\183\000\
\184\000\185\000\186\000\189\000\193\000\197\000\198\000\199\000\
\067\000\077\000\091\000\054\000\057\000\016\000\015\000\119\000\
\162\000\177\000\126\000\156\000\063\000\116\000\120\000\106\000\
\108\000\058\000\078\000\064\000\081\000"

let yycheck = "\039\000\
\040\000\015\000\066\000\017\000\129\000\019\000\105\000\076\000\
\009\001\002\001\096\000\002\001\137\000\136\000\017\001\029\000\
\001\001\001\001\021\001\002\001\145\000\146\000\147\000\037\000\
\149\000\111\000\112\000\020\001\151\000\020\001\002\001\015\001\
\016\001\017\001\018\001\049\000\020\001\020\001\001\001\001\001\
\001\001\008\001\017\001\083\000\003\001\014\001\115\000\017\001\
\020\001\017\001\119\000\002\001\177\000\176\000\015\001\016\001\
\017\001\018\001\001\001\020\001\001\000\002\000\003\000\162\000\
\187\000\190\000\191\000\192\000\154\000\020\001\002\001\013\001\
\023\001\024\001\025\001\026\001\027\001\028\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\036\001\037\001\038\001\
\128\000\020\001\001\001\155\000\001\001\004\001\013\001\026\001\
\001\001\001\001\001\001\010\001\013\001\032\001\001\001\034\001\
\002\001\004\001\017\001\038\001\019\001\001\001\150\000\010\001\
\015\001\016\001\017\001\018\001\013\001\013\001\017\001\159\000\
\019\001\004\001\005\001\001\001\007\001\001\001\140\000\010\001\
\011\001\012\001\002\001\015\001\016\001\017\001\017\001\018\001\
\019\001\021\001\021\001\020\001\013\001\001\001\023\001\022\001\
\184\000\185\000\027\001\001\001\029\001\030\001\031\001\017\001\
\033\001\034\001\004\001\005\001\037\001\007\001\002\001\001\001\
\010\001\011\001\012\001\002\001\002\001\002\001\001\001\017\001\
\020\001\019\001\001\001\021\001\024\001\025\001\020\001\002\001\
\028\001\006\001\002\001\002\001\002\001\002\001\034\001\035\001\
\036\001\019\001\015\001\016\001\017\001\018\001\018\001\002\001\
\021\001\018\001\130\000\002\001\002\001\002\001\002\001\002\001\
\002\001\002\001\002\001\002\001\002\001\002\001\002\001\002\001\
\002\001\002\001\002\001\002\001\002\001\002\001\002\001\090\000\
\130\000\161\000\100\000\128\000\034\000\088\000\092\000\075\000\
\077\000\029\000\044\000\037\000\049\000"

let yynames_const = "\
  LEFT_PAREN\000\
  RIGHT_PAREN\000\
  BENCHMARK\000\
  FALSE\000\
  FLET\000\
  ITE\000\
  LET\000\
  LOGIC\000\
  THEORY\000\
  TRUE\000\
  ATTRIBUTE_ASSUMPTION\000\
  ATTRIBUTE_AXIOMS\000\
  ATTRIBUTE_DEFINITION\000\
  ATTRIBUTE_EXTENSIONS\000\
  ATTRIBUTE_FORMULA\000\
  ATTRIBUTE_FUNS\000\
  ATTRIBUTE_EXTRAFUNS\000\
  ATTRIBUTE_EXTRASORTS\000\
  ATTRIBUTE_EXTRAPREDS\000\
  ATTRIBUTE_LANGUAGE\000\
  ATTRIBUTE_LOGIC\000\
  ATTRIBUTE_NOTES\000\
  ATTRIBUTE_PREDS\000\
  ATTRIBUTE_SORTS\000\
  ATTRIBUTE_STATUS\000\
  ATTRIBUTE_THEORY\000\
  "

let yynames_block = "\
  CONNECTIVE\000\
  QUANT_SYMB\000\
  STRING\000\
  USER_VALUE\000\
  NUMERAL\000\
  RATIONAL\000\
  IDENTIFIER\000\
  VAR\000\
  FVAR\000\
  ATTRIBUTE\000\
  ARITH_SYMB\000\
  STATUS\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 130 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 378 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 131 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 385 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 135 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 392 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 136 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 399 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 140 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 406 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 144 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
            ( [_1] )
# 413 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 145 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                               ( _2::_1 )
# 421 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string list) in
    Obj.repr(
# 149 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                     ( List.rev _1 )
# 428 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string list))
; (fun __caml_parser_env ->
    Obj.repr(
# 153 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
  ( [] )
# 434 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string list) in
    Obj.repr(
# 154 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                 ( _1 )
# 441 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 158 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
            ( (_1, None) )
# 448 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string * (string option)))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 159 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                       ( (_1, Some _2) )
# 456 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string * (string option)))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string * (string option)) in
    Obj.repr(
# 163 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( [_1] )
# 463 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (string * (string option)) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string * (string option)) in
    Obj.repr(
# 164 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                 ( _2::_1 )
# 471 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (string * (string option)) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : (string * (string option)) list) in
    Obj.repr(
# 168 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                      ( List.rev _1 )
# 478 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (string * (string option)) list))
; (fun __caml_parser_env ->
    Obj.repr(
# 172 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
  ( [] )
# 484 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (string * (string option)) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : (string * (string option)) list) in
    Obj.repr(
# 173 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                  ( _1 )
# 491 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (string * (string option)) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 177 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
      ( Term.variable (variable _1) )
# 498 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 178 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
          ( Term.constant (integer _1) )
# 505 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 179 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
           ( Term.constant (rational _1) )
# 512 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 180 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Term.constant (constant _1) )
# 519 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term) in
    Obj.repr(
# 184 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
          ( [_1] )
# 526 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Term.term list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Term.term) in
    Obj.repr(
# 185 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                           ( _2::_1 )
# 534 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term list) in
    Obj.repr(
# 189 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                   ( List.rev _1 )
# 541 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term) in
    Obj.repr(
# 193 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
            ( _1 )
# 548 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Term.term) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 194 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                   ( _2 )
# 556 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : Term.term list) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 195 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                               ( 
    application _2 _3)
# 566 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 4 : (Variable.t * Term.term) list * Term.term) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : Term.term) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : Term.term) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 197 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                        ( 
    Term_logic.Standard.ite (bind_local _3) _4 _5 )
# 577 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    Obj.repr(
# 204 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Term_constant.truth )
# 583 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    Obj.repr(
# 205 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Term_constant.falsity )
# 589 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 206 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Term.variable (variable _1) )
# 596 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 207 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Term.constant (constant _1) )
# 603 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term) in
    Obj.repr(
# 211 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
            ( _1 )
# 610 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : Term.term) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 212 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                   ( _2 )
# 618 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : Term.term list) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 213 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                ( 
    predicate _2 _3 )
# 628 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : (Variable.t * Term.term) list * Term.term) in
    Obj.repr(
# 218 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( [bind_local _1] )
# 635 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Term.term list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : (Variable.t * Term.term) list * Term.term) in
    Obj.repr(
# 219 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                 ( (bind_local _2)::_1 )
# 643 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term list) in
    Obj.repr(
# 223 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                      ( List.rev _1 )
# 650 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Term.term list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 227 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                       ( Variable.named _2 )
# 658 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Variable.t))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Variable.t) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : (Variable.t * Term.term) list * Term.term) in
    Obj.repr(
# 232 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                       ( ([_1], bind_local _2) )
# 666 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Variable.t list * Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Variable.t) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Variable.t list * Term.term) in
    Obj.repr(
# 233 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                      ( let (vs,f) = _2 in (_1::vs, f) )
# 674 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Variable.t list * Term.term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Term.term) in
    Obj.repr(
# 237 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
          ( ([],_1) )
# 681 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (Variable.t * Term.term) list * Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : Term.term list) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 238 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                    ( 
    ([], connection _2 _3) )
# 691 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (Variable.t * Term.term) list * Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : Variable.t list * Term.term) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 240 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                              (
    let (vars,body) = _3 in ([], quantification _2 vars body) )
# 701 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (Variable.t * Term.term) list * Term.term))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 4 : Term.term) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : (Variable.t * Term.term) list * Term.term) in
    let _8 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 244 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
              (
    let (bindings,body) = _7 in
      ((Variable.named _4, _5)::bindings, body) )
# 713 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (Variable.t * Term.term) list * Term.term))
; (fun __caml_parser_env ->
    let _4 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 4 : (Variable.t * Term.term) list * Term.term) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : (Variable.t * Term.term) list * Term.term) in
    let _8 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 249 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
              ( 
    let (bindings,body) = _7 in
      ((Variable.named _4, bind_local _5)::bindings, body) )
# 725 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : (Variable.t * Term.term) list * Term.term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string list) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 257 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                 ( 
    declare_constant _2 _3 )
# 735 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 259 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                           ( 
    (integer _2) )
# 745 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 261 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                            ( 
    (rational _2) )
# 755 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string list) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : (string * (string option)) list) in
    Obj.repr(
# 266 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                  ( 
    declare_predicate _2 _3 )
# 765 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 271 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 772 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Constant.t) in
    Obj.repr(
# 275 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                ( [_1] )
# 779 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Constant.t) in
    Obj.repr(
# 276 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                       ( _2::_1 )
# 787 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Constant.t list) in
    Obj.repr(
# 280 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                         ( List.rev _1 )
# 794 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Constant.t) in
    Obj.repr(
# 284 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                 ( [_1] )
# 801 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Constant.t) in
    Obj.repr(
# 285 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                         ( _2::_1 )
# 809 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Constant.t list) in
    Obj.repr(
# 289 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                          ( List.rev _1 )
# 816 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Constant.t list))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string list) in
    Obj.repr(
# 293 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                        ( Sorts _3 )
# 823 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    Obj.repr(
# 294 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                           ( Funs _3 )
# 830 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    Obj.repr(
# 295 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                             ( Preds _3 )
# 837 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 296 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                              ( Definition _2 )
# 844 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Term.term list) in
    Obj.repr(
# 297 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                          ( Axioms _3 )
# 851 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 298 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                         ( Notes _2 )
# 858 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string * (string option)) in
    Obj.repr(
# 299 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Annotation _1 )
# 865 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 303 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                   ( [_1] )
# 872 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 304 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                             ( _2::_1 )
# 880 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 308 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                            ( List.rev _1 )
# 887 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 313 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
    ( (Name _3)::_4 )
# 895 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 319 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 902 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 323 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                               ( 
    let file = Smtlib_parser.smtlib_theory_file _2 in
      Theory (Smtlib_parser.parse_theory file) )
# 911 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 326 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                            ( Language _2 )
# 918 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 327 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                              ( Extensions _2 )
# 925 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 328 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                         ( Notes _2 )
# 932 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string * (string option)) in
    Obj.repr(
# 329 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( Annotation _1 )
# 939 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 333 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                  ( [_1] )
# 946 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 334 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                           ( _2::_1 )
# 954 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 338 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                           ( List.rev _1 )
# 961 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 343 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
    ( (Name _3)::_4 )
# 969 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 349 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             ( _1 )
# 976 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : string))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 353 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                             (
    let file = Smtlib_parser.smtlib_logic_file _2 in
      Logic (Smtlib_parser.parse_logic file) )
# 985 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : (Variable.t * Term.term) list * Term.term) in
    Obj.repr(
# 356 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                  ( 
    Assumption (bind_local _2) )
# 993 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : (Variable.t * Term.term) list * Term.term) in
    Obj.repr(
# 358 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                               ( 
    Formula (bind_local _2) )
# 1001 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.status) in
    Obj.repr(
# 360 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                          ( 
    Status _2 )
# 1009 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string list) in
    Obj.repr(
# 362 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                             ( Sorts _3 )
# 1016 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    Obj.repr(
# 363 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                ( 
    Funs _3 )
# 1024 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : Constant.t list) in
    Obj.repr(
# 365 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                  ( 
    Preds _3 )
# 1032 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 367 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                         ( 
    Notes _2 )
# 1040 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string * (string option)) in
    Obj.repr(
# 369 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
             (
    Annotation _1 )
# 1048 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 374 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                  ( [_1] )
# 1055 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute) in
    Obj.repr(
# 375 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                           ( _2::_1 )
# 1063 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 379 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                           ( List.rev _1 )
# 1070 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : Smtlib_attribute.attribute list) in
    Obj.repr(
# 383 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.mly"
                                                                   ( 
    (Name _3)::_4 )
# 1079 "/u/f6/rupak/blast/psrc2/fe/dp/dpt/dpt-1.1/src/smtlib/smtlib_grammar.ml"
               : Smtlib_attribute.attribute list))
(* Entry theory *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry logic *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
(* Entry benchmark *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let theory (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Smtlib_attribute.attribute list)
let logic (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 2 lexfun lexbuf : Smtlib_attribute.attribute list)
let benchmark (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 3 lexfun lexbuf : Smtlib_attribute.attribute list)
