(* $Id: cc_literal.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Literal representation for Congruence Closure. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)


(** Variables are represented by unique identifiers.

    Treat this as if Id.t was abstract, but if we make it so then the
    compiler generates slower code. *)
module Variable: Id.Generator with type t = int

(** The type of (dis)equality literals. *)
type literal = 
  | Equality of Variable.t * Variable.t
  | Disequality of Variable.t * Variable.t

val invalid_literal: literal
  (** An invalid literal. *)

val is_valid_literal: literal -> bool
  (** Is a literal valid. *)

val equal: literal -> literal -> bool
  (* [equal l m] literals [l] and [m] equal (understands that equality
     and disequality are commutative)?
     
     - Require {!is_valid_literal}[ l]
     - Require {!is_valid_literal}[ m] *)

val negation: literal -> literal
  (** [negation l] is the nation of [l].

      - Require {!is_valid_literal}[ l]
      - Ensure {!is_valid_literal}[ l] 
      - Ensure [not (]{!equal}[ l negation l)] *)

module Variable_map: Index.Map with type key = Variable.t
module Variable_hash: Hashtbl.S with type key = Variable.t
module Variable_map2: Index.Map2
  with type key0 = Variable.t and type key1 = Variable.t
module Variable_set: Index.Set with type elt = Variable.t
