(* $Id: dpll_literal.ml,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(* Literals and variables. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

module Variable       = Id.Nat_generator
module Variable_map   = Index.Nat_map
module Variable_set   = Index.Nat_set
module Variable_queue = Index.Nat_priority_queue

module Literal        = Id.Nat
module Literal_map    = Index.Nat_map
module Literal_set    = Index.Nat_set

(* l div 2 *)
let variable l   = 
  assert (Literal.is_valid l);
  l lsr 1
  
(* l mod 2 = 0 : is l even? *)
let is_positive l   = 
  assert (Literal.is_valid l);
  (l land 1) = 0
  
(* 2v *)
let literal v  = 
  assert (Variable.is_valid v);
  let p = v lsl 1 in
    assert (is_positive p);
    assert (variable p = v);
    p

(* l xor 1 : even <-> odd conversion *)
let negation l =
  assert (Literal.is_valid l);
  let m = l lxor 1 in
    assert (is_positive m = not (is_positive l));
    assert (variable m = variable l);
    m
