(* $Id: constant.ml,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(* Constants. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Big_int
open Num

type t = 
  | Integer  of big_int
  | Rational of num 
  | Symbolic of Symbol.t

let equal c d =
  match (c, d) with
    | (Integer  x, Integer  y) -> eq_big_int x y
    | (Rational x, Rational y) -> eq_num x y
    | (Symbolic x, Symbolic y) -> Symbol.equal x y
    | _ -> false

let compare c d =
  match (c, d) with
    | (Integer  x, Integer  y) -> compare_big_int x y
    | (Rational x, Rational y) -> compare_num x y
    | (Symbolic x, Symbolic y) -> Symbol.compare x y
    | (Integer  _, _         ) -> -1
    | (_,          Integer  _) ->  1
    | (Rational _, Symbolic _) -> -1
    | (Symbolic _, Rational _) ->  1

let hash_big_int x =
  let big_int_of_max_int = big_int_of_int Pervasives.max_int in
  let x_mod_max_int =
    if is_int_big_int x then 
      x
    else 
      mod_big_int x big_int_of_max_int
  in
    int_of_big_int x_mod_max_int

let hash_num x = Hashtbl.hash (float_of_num x)

let hash = function
  | Integer  x -> hash_big_int x
  | Rational x -> hash_num x
  | Symbolic x -> Symbol.hash x

let integer i = Integer i

let is_integer = function
  | Integer _ -> true
  | _         -> false

let integer_value = function
  | Integer i -> i
  | _         -> assert false

let rational r = Rational r

let is_rational = function
  | Rational _ -> true
  | _          -> false

let rational_value = function
  | Rational r -> r
  | _          -> assert false

let symbolic s = Symbolic s

let is_symbolic = function
  | Symbolic _ -> true
  | _          -> false

let symbolic_value = function
  | Symbolic s -> s
  | _          -> assert false

let arity = function
  | Integer _ -> 0
  | Rational _ -> 0
  | Symbolic s -> Symbol.arity s

let string_of = function
  | Integer i  -> string_of_big_int i
  | Rational r -> string_of_num r
  | Symbolic s -> Symbol.string_of_symbol s

let pp_print f = function
  | Integer i  -> Format.pp_print_string f (string_of_big_int i)
  | Rational r -> Format.pp_print_string f (string_of_num r)
  | Symbolic s -> Symbol.pp_print f s

let log level c = Log.log pp_print level c
