(* $Id: symbol.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Symbols. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** Abstract type of symbols. *)
type t

val equal: t -> t -> bool
  (** Are two symbols equal? *)

val hash: t -> int
  (** Hash code of a symbol. *)

val compare: t -> t -> int
  (** An arbitrary ordering on symbols. *)

val name_declared: string -> bool
  (** [name_declared n] is true when a symbol has been declared with name n. *)

val declare: string -> int -> t
  (** [declare n a] declares a new symbol with the name [n] and arity [a]
      which is interpreted by some theory.
      
      - Require [a >= 0]
      - Require [not (name_declared n)] *)

val declare_uninterpreted: string -> int -> t
  (** like [declare], but for an uninterpreted symbol *)

val find: string -> t
  (** [find n] is the symbol declared with name [n].

      @raise Not_found when [not (name_declared n)] *)

val arity: t -> int
  (** [arity s] is the arity declared for [s]. *)

val interpreted: t -> bool

val string_of_symbol: t-> string
  (** String representation (name) of a symbol. *)

val pp_print: Format.formatter -> t -> unit
  (** Pretty print a symbol. *)

module Primitive: sig
  val truth: t
  val falsity: t
  val equality: t
  val negation: t
  val disjunction: t
  val conjunction: t
  val difference: t
  val equivalence: t
  val implication: t
  val conditional:t 
  val ite:t 
  val less: t
  val less_or_equal: t
  val greater: t
  val greater_or_equal: t
  val unary_minus: t
  val plus: t
  val minus: t
  val times: t
  val divide: t
end

