(* $Id: term_literal.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Term literals. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

open Term

val is_literal: term -> bool
  (** Holds when a term is a variable or the negation of a variable. 

     - Ensure [not (]{!Term.is_variable}[ t) || (is_literal l)] *)
    
val variable: term -> term
  (** [variable l] is the variable contained in a literal term. 
      
      - Require {!is_literal}[ l]
      - Ensure {!Term.is_variable}[ (variable l)] *)

val is_positive: term -> bool
  (** [is_positive l] holds if the literal term [t] is positive.

      - Require {!is_literal}[ l] *)
    
val literal: term -> term
  (** [literal v] is the positive literal term with variable [v]. 
     
     - Require {!Term.is_variable}[ v] 
     - Ensure {!is_positive }[(literal v)]
     - Ensure {!variable}[ (literal v) = v] *)
  
val negation: term -> term
  (** Negation of a term literal. 

     - Require [is_literal l]
     - Ensure [is_positive (negation l) = not (is_positive l)] 
     - Ensure [variable (negation l) = variable l] *)




