(* $Id: variable.mli,v 1.1.1.1 2008/05/22 16:46:07 rupak Exp $ *)
(** Variables. *)

(* Copyright 2007 Intel Corporation 

   Licensed under the Apache License, Version 2.0 (the "License"); you
   may not use this file except in compliance with the License.  You
   may obtain a copy of the License at
   
   http://www.apache.org/licenses/LICENSE-2.0
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
   implied.  See the License for the specific language governing
   permissions and limitations under the License. *)

(** Variables *)
type t

val named: string -> t
  (** [named n] variable with the name [n]. 

      - Require [n != ""] *)

val fresh: unit -> t
  (** A fresh variable. *)

val variant: t list -> t -> t
  (** [variant avoid v] is a variable with a name that of [v], but not
      occurring in [avoid]. *)

val equal: t -> t -> bool
  (** Are two variables equal? *)

val hash: t -> int
  (** Hash code of a variable. *)

val compare: t -> t -> int
  (** An arbitrary ordering on variables. *)

val string_of: t -> string
  (** string representation of a variable. *)

val pp_print: Format.formatter -> t -> unit
  (** Print a variable. *)

val log: int -> t -> unit
  (** [log level c] is equivalent to 
      [Log.log level pp_print c] *)
