#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/custom.h>
#include "yices_c.h"
#include "yicesl_c.h"

extern yicesl_context yices_get_lite_context(yices_context ctx);

#define Expr_val(v) (* ((yices_expr **) Data_custom_val(v)))
#define Type_val(v) (* ((yices_type **) Data_custom_val(v)))
#define Var_decl_val(v) (* ((yices_var_decl **) Data_custom_val(v)))
#define Context_val(v) (* ((yices_context **) Data_custom_val(v)))
#define Lcontext_val(v) (* ((yicesl_context **) Data_custom_val(v)))
#define Assertion_id_val(v) Int_val(v)
#define Val_assertion_id_val(v) Val_int(v)
#define Model_val(v) (* ((yices_model **) Data_custom_val(v)))
#define Var_decl_iterator_val(v) (* ((yices_var_decl_iterator **) Data_custom_val(v)))

#define Val_lbool(v) \
    (v == l_false) \
    ? hash_variant("Lfalse") \
    : ((v == l_undef) \
        ? hash_variant("Lundef") \
	: hash_variant("Ltrue")) 

#define Convert_list_to_array(lst, iter, arr, len, typ, val) \
    int iter, len; \
    yices_expr *arr; \
    len = 0; \
    iter = lst; \
    while ( iter && Is_block(iter) ) { len++; iter = Field(iter, 1); } \
    arr = (typ *) malloc( len * sizeof( typ )); \
    len = 0; \
    iter = lst; \
    while ( iter && Is_block(iter) ) { \
	arr[len++] = val( Field(iter, 0) ); \
	iter = Field(iter, 1); \
    }

static value alloc_expr(yices_expr * e) {
    value v = caml_alloc(sizeof(yices_expr *), 0);
    Expr_val(v) = e;
    return v;
}

static value alloc_type(yices_type * t) {
    value v = caml_alloc(sizeof(yices_type *), 0);
    Type_val(v) = t;
    return v;
}

static value alloc_var_decl(yices_var_decl * d) {
    value v = caml_alloc(sizeof(yices_var_decl *), 0);
    Var_decl_val(v) = d;
    return v;
}

static value alloc_context(yices_context * c) {
    value v = caml_alloc(sizeof(yices_context *), 0);
    Context_val(v) = c;
    return v;
}

static value alloc_lcontext(yicesl_context * c) {
    value v = caml_alloc(sizeof(yicesl_context *), 0);
    Lcontext_val(v) = c;
    return v;
}

static value alloc_model(yices_model * m) {
    value v = caml_alloc(sizeof(yices_model *), 0);
    Model_val(v) = m;
    return v;
}

value ml_yices_set_verbosity(value l) {
    CAMLparam1(l);
    yices_set_verbosity(Int_val(l));
    CAMLreturn( Val_unit );
}

// char *  yices_version ()
// void  yices_set_max_num_conflicts_in_maxsat_iteration (unsigned n)
// void  yices_enable_type_checker (int flag)
// void  yices_set_max_num_iterations_in_maxsat (unsigned n)
// void  yices_set_maxsat_initial_cost (long long c)
// void  yices_set_arith_only (int flag)

value ml_yices_enable_log_file(value file_name) {
    CAMLparam1(file_name);
    yices_enable_log_file(String_val(file_name));
    CAMLreturn( Val_unit );
}

value ml_yices_mk_context(value unit) {
    CAMLparam1(unit);
    CAMLreturn( alloc_context(yices_mk_context()) );
}

value ml_yices_del_context(value ctx) {
    CAMLparam1(ctx);
    yices_del_context(Context_val(ctx));
    CAMLreturn( Val_unit );
}

value ml_yices_reset(value ctx) {
    CAMLparam1(ctx);
    yices_reset(Context_val(ctx));
    CAMLreturn( Val_unit );
}

value ml_yices_dump_context(value ctx) {
    CAMLparam1(ctx);
    yices_dump_context(Context_val(ctx));
    CAMLreturn( Val_unit );
}

value ml_yices_push(value ctx) {
    CAMLparam1(ctx);
    yices_push(Context_val(ctx));
    CAMLreturn( Val_unit );
}

value ml_yices_pop(value ctx) {
    CAMLparam1(ctx);
    yices_pop(Context_val(ctx));
    CAMLreturn( Val_unit );
}

value ml_yices_assert(value ctx, value expr) {
    CAMLparam2(ctx, expr);
    yices_assert(Context_val(ctx), Expr_val(expr));
    CAMLreturn( Val_unit );
}

// assertion_id  yices_assert_weighted (yices_context ctx, yices_expr expr, long long w)
// assertion_id  yices_assert_retractable (yices_context ctx, yices_expr expr)

value ml_yices_retract(value ctx, value id) {
    CAMLparam2(ctx, id);
    yices_retract(Context_val(ctx), Assertion_id_val(id));
    CAMLreturn( Val_unit );
}

value ml_yices_inconsistent(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn(Val_int( yices_inconsistent(Context_val(ctx)) ));
}

value ml_yices_check(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( Val_int(yices_check(Context_val(ctx))) );
}

// lbool  yices_find_weighted_model (yices_context ctx, int random)
// lbool  yices_evaluate_in_model (yices_model m, yices_expr e)
// lbool  yices_max_sat (yices_context ctx)
// lbool  yices_max_sat_cost_leq (yices_context c, long long max_cost)

value ml_yices_get_model(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( alloc_model(yices_get_model(Context_val(ctx))) );
}

value ml_yices_get_value(value m, value v) {
    CAMLparam2(m, v);
    CAMLreturn( Val_int(yices_get_value(Model_val(m), Var_decl_val(v))) );
}

value ml_yices_get_int_value(value m, value d) {
    CAMLparam2(m, d);
    CAMLlocal3(result, v, r);
    r = yices_get_int_value( Model_val(m), Var_decl_val(d), &v );

    result = caml_alloc_tuple(2);
    Store_field(result, 0, Val_int(r));
    Store_field(result, 1, Val_long(v));
    CAMLreturn(result);
}

// int  yices_get_arith_value (yices_model m, yices_var_decl d, long *num, long *den)
// int  yices_get_double_value (yices_model m, yices_var_decl d, double *value)
// int  yices_get_mpq_value (yices_model m, yices_var_decl d, mpq_t value)
// int  yices_get_mpz_value (yices_model m, yices_var_decl d, mpz_t value)
// int  yices_get_bitvector_value (yices_model m, yices_var_decl d, unsigned n, int bv[])
// int  yices_get_assertion_value (yices_model m, assertion_id id)

value ml_yices_display_model(value m) {
    CAMLparam1(m);
    yices_display_model( Model_val(m) );
    CAMLreturn( Val_unit );
}

// long long  yices_get_cost (yices_model m)
// double  yices_get_cost_as_double (yices_model m)

value ml_yices_mk_true(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( alloc_expr(yices_mk_true(Context_val(ctx))) );
}
value ml_yices_mk_false(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( alloc_expr(yices_mk_false(Context_val(ctx))) );
}
value ml_yices_mk_bool_var(value ctx, value name) {
    CAMLparam2(ctx, name);
    CAMLreturn( alloc_expr(yices_mk_bool_var(Context_val(ctx), String_val(name))) );
}
value ml_yices_mk_fresh_bool_var(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( alloc_expr(yices_mk_fresh_bool_var(Context_val(ctx))) );
}

value ml_yices_get_var_decl(value e) {
    CAMLparam1(e);
    CAMLreturn( alloc_var_decl(yices_get_var_decl(Expr_val(e))) );
}
value ml_yices_mk_bool_var_decl(value ctx, value name) {
    CAMLparam2(ctx, name);
    CAMLreturn( alloc_var_decl(yices_mk_bool_var_decl(Context_val(ctx), String_val(name))) );
}
value ml_yices_get_var_decl_name(value d) {
    CAMLparam1(d);
    CAMLreturn( caml_copy_string(yices_get_var_decl_name(Var_decl_val(d))) );
}
value ml_yices_mk_bool_var_from_decl(value ctx, value d) {
    CAMLparam2(ctx, d);
    CAMLreturn( alloc_expr(yices_mk_bool_var_from_decl(Context_val(ctx), Var_decl_val(d))) );
}

value ml_yices_mk_or(value ctx, value ls) {
    CAMLparam2(ctx, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_or(Context_val(ctx), args, n)) );
}
value ml_yices_mk_and(value ctx, value ls) {
    CAMLparam2(ctx, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_and(Context_val(ctx), args, n)) );
}

value ml_yices_mk_eq(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_eq(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_diseq(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_diseq(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_ite(value ctx, value c, value t, value e) {
    CAMLparam4(ctx, c, t, e);
    CAMLreturn( alloc_expr(yices_mk_ite(Context_val(ctx), Expr_val(c), Expr_val(t), Expr_val(e))) );
}
value ml_yices_mk_not(value ctx, value a) {
    CAMLparam2(ctx, a);
    CAMLreturn( alloc_expr(yices_mk_not(Context_val(ctx), Expr_val(a))) );
}

// yices_var_decl_iterator yices_create_var_decl_iterator (yices_context c)
// int  yices_iterator_has_next (yices_var_decl_iterator it)
// yices_var_decl  yices_iterator_next (yices_var_decl_iterator it)
// void  yices_iterator_reset (yices_var_decl_iterator it)
// void  yices_del_iterator (yices_var_decl_iterator it)

value ml_yices_mk_type(value ctx, value name) {
    CAMLparam2(ctx, name);
    CAMLreturn( alloc_type(yices_mk_type(Context_val(ctx), String_val(name))) );
}

value ml_yices_mk_function_type(value ctx, value domain, value range) {
    CAMLparam3(ctx, domain, range);
    Convert_list_to_array(domain, i, args, n, yices_type, Type_val);
    CAMLreturn( alloc_type(yices_mk_function_type(Context_val(ctx), args, n, Type_val(range))) );
}

// yices_type  yices_mk_bitvector_type (yices_context ctx, unsigned size)
// yices_type  yices_mk_tuple_type (yices_context ctx, yices_type *args[], unsigned size)

value ml_yices_mk_var_decl(value ctx, value name, value ty) {
    CAMLparam3(ctx, name, ty);
    CAMLreturn( alloc_var_decl(yices_mk_var_decl(Context_val(ctx), String_val(name), Type_val(ty))) );
}
value ml_yices_get_var_decl_from_name(value ctx, value name) {
    CAMLparam2(ctx, name);
    CAMLreturn( alloc_var_decl(yices_get_var_decl_from_name(Context_val(ctx), String_val(name))) );
}
value ml_yices_mk_var_from_decl(value ctx, value d) {
    CAMLparam2(ctx, d);
    CAMLreturn( alloc_expr(yices_mk_var_from_decl(Context_val(ctx), Var_decl_val(d))) );
}

value ml_yices_mk_app(value ctx, value f, value ls) {
    CAMLparam3(ctx, f, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_app(Context_val(ctx), Expr_val(f), args, n)) );
}
value ml_yices_mk_num(value ctx, value n) {
    CAMLparam2(ctx, n);
    CAMLreturn( alloc_expr(yices_mk_num(Context_val(ctx), Int_val(n))) );
}
value ml_yices_mk_num_from_string(value ctx, value n) {
    CAMLparam2(ctx, n);
    CAMLreturn( alloc_expr(yices_mk_num_from_string(Context_val(ctx), String_val(n))) );
}

// yices_expr  yices_mk_num_from_mpz (yices_context ctx, const mpz_t z)
// yices_expr  yices_mk_num_from_mpq (yices_context ctx, const mpq_t q)

value ml_yices_mk_sum(value ctx, value ls) {
    CAMLparam2(ctx, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_sum(Context_val(ctx), args, n)) );
}
value ml_yices_mk_sub(value ctx, value ls) {
    CAMLparam2(ctx, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_sub(Context_val(ctx), args, n)) );
}
value ml_yices_mk_mul(value ctx, value ls) {
    CAMLparam2(ctx, ls);
    Convert_list_to_array(ls, i, args, n, yices_expr, Expr_val);
    CAMLreturn( alloc_expr(yices_mk_mul(Context_val(ctx), args, n)) );
}

value ml_yices_mk_lt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_lt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_le(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_le(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_gt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_gt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_ge(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_ge(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}

// yices_expr  yices_mk_bv_constant (yices_context ctx, unsigned size, unsigned long value)
// yices_expr  yices_mk_bv_constant_from_array (yices_context ctx, unsigned size, int bv[])

value ml_yices_mk_bv_add(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_add(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_sub(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_sub(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_mul(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_mul(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_minus(value ctx, value a1) {
    CAMLparam2(ctx, a1);
    CAMLreturn( alloc_expr(yices_mk_bv_minus(Context_val(ctx), Expr_val(a1))) );
}
value ml_yices_mk_bv_concat(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_concat(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_and(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_and(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_or(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_or(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_xor(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_xor(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}

// yices_expr  yices_mk_bv_not (yices_context ctx, yices_expr a1)
// yices_expr  yices_mk_bv_extract (yices_context ctx, unsigned end, unsigned begin, yices_expr a)
// yices_expr  yices_mk_bv_sign_extend (yices_context ctx, yices_expr a, unsigned n)
// yices_expr  yices_mk_bv_shift_left0 (yices_context ctx, yices_expr a, unsigned n)
// yices_expr  yices_mk_bv_shift_left1 (yices_context ctx, yices_expr a, unsigned n)
// yices_expr  yices_mk_bv_shift_right0 (yices_context ctx, yices_expr a, unsigned n)
// yices_expr  yices_mk_bv_shift_right1 (yices_context ctx, yices_expr a, unsigned n)

value ml_yices_mk_bv_lt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_lt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_le(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_le(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_gt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_gt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_ge(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_ge(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_slt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_slt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_sle(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_sle(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_sgt(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_sgt(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}
value ml_yices_mk_bv_sge(value ctx, value a1, value a2) {
    CAMLparam3(ctx, a1, a2);
    CAMLreturn( alloc_expr(yices_mk_bv_sge(Context_val(ctx), Expr_val(a1), Expr_val(a2))) );
}

value ml_yices_pp_expr(value e) {
    CAMLparam1(e);
    yices_pp_expr(Expr_val(e));
    CAMLreturn( Val_unit );
}

value ml_yices_get_lite_context(value ctx) {
    CAMLparam1(ctx);
    CAMLreturn( alloc_lcontext(yices_get_lite_context(Context_val(ctx))) );
}
